package datavisualizer;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.io.Serializable;

import org.jfree.chart.CrosshairInfo;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.entity.XYItemEntity;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.XYDataset;
import org.jfree.data.XYZDataset;
import org.jfree.ui.RectangleEdge;
import org.jfree.util.PublicCloneable;
import org.jfree.chart.renderer.AbstractXYItemRenderer;
import org.jfree.chart.renderer.XYItemRenderer;

/**
 * A renderer that draws a circle at each data point.  The renderer expects
 * the dataset to be an XYZDataset.
 *
 * Original author of XYBubbleRenderer, upon which this is based - David
 * Gilbert
 * <p>Title: Data Visualizer</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>May 9, 2006:  Added support for a minimum display DBH (LEM)
 */
public class XYTreeRenderer
    extends AbstractXYItemRenderer
    implements XYItemRenderer,
    Cloneable,
    PublicCloneable,
    Serializable {

  /**Controls how large trees appear on the screen*/
  private double m_fScaleFactor = 0.25;

  /**The minimum DBH (z) value to display*/
  private double m_fMinDbh = 0;

  /**
   * Constructs a new renderer.
   */
  public XYTreeRenderer() {
    super();
  }

  /**
   * Sets the scale factor, which controls how large trees appear on the
   * screen.
   * @param fScaleFactor double Scale factor.
   */
  public void SetScaleFactor(double fScaleFactor) {
    m_fScaleFactor = fScaleFactor;
  }

  /**
   * Gets the scale factor, which controls how large trees appear on the
   * screen.
   * @return Scale factor.
   */
  public double GetScaleFactor() {
    return m_fScaleFactor;
  }

  /**
   * Set the value for minimum DBH.
   * @param fMinDbh double Value for minimum DBH.
   */
  public void SetMinDbh(double fMinDbh) {
    m_fMinDbh = fMinDbh;
  }

  /**
   * Gets the current value for minimum DBH.
   * @return double The minimum DBH.
   */
  public double GetMinDbh() {
    return m_fMinDbh;
  }

  /**
   * Draws the visual representation of a single data item.
   *
   * @param g2  the graphics device.
   * @param dataArea  the area within which the data is being drawn.
   * @param info  collects information about the drawing.
   * @param plot  the plot (can be used to obtain standard color information
   * etc).
   * @param domainAxis  the domain (horizontal) axis.
   * @param rangeAxis  the range (vertical) axis.
   * @param dataset  the dataset.
   * @param series  the series index (zero-based).
   * @param item  the item index (zero-based).
   * @param crosshairInfo  information about crosshairs on a plot.
   * @param pass  the pass index.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void drawItem(Graphics2D g2,
                       Rectangle2D dataArea,
                       PlotRenderingInfo info,
                       XYPlot plot,
                       ValueAxis domainAxis,
                       ValueAxis rangeAxis,
                       XYDataset dataset,
                       int series,
                       int item,
                       CrosshairInfo crosshairInfo,
                       int pass) {

    // get the data point...
    Number xn = dataset.getXValue(series, item);
    Number yn = dataset.getYValue(series, item);
    Number zn = null;
    if (dataset instanceof XYZDataset) {
      XYZDataset xyzData = (XYZDataset) dataset;
      zn = xyzData.getZValue(series, item);
    }
    if (zn != null) {
      double x = xn.doubleValue();
      double y = yn.doubleValue();
      double z = zn.doubleValue();

      //Make sure the DBH (z) value is above our minimum
      if (z < m_fMinDbh) return;

      //Get the edges of the plot
      RectangleEdge domainAxisLocation = plot.getDomainAxisEdge();
      RectangleEdge rangeAxisLocation = plot.getRangeAxisEdge();

      //Translate the coordinates from XY to the Java2D coordinates where
      //the origin is at the upper right corner of the plot
      double transX = domainAxis.translateValueToJava2D(x, dataArea,
          domainAxisLocation);
      double transY = rangeAxis.translateValueToJava2D(y, dataArea,
          rangeAxisLocation);

      //Calculate the radius of our circle - this sizes it about right
      //Note that this is a simplification that only works if the chart is
      //sized according to the plot lengths
      double ellipseRadius = z * m_fScaleFactor; //radius of circle

      //Create our ellipse
      Ellipse2D circle = null;
      if (plot.getOrientation() == PlotOrientation.VERTICAL) {

        circle = new Ellipse2D.Double(transX - ellipseRadius, //X coordinate of rectangle enclosing ellipse
                                      transY - ellipseRadius, //Y coordinate of rectangle enclosing ellipse
                                      //ellipseRadius * 2, //width of ellipse
                                      //ellipseRadius * 2); //height of ellipse
                                      ellipseRadius, //width of ellipse
                                      ellipseRadius); //height of ellipse
      }
      else if (plot.getOrientation() == PlotOrientation.HORIZONTAL) {
        circle = new Ellipse2D.Double(transY - ellipseRadius,
                                      transX - ellipseRadius,
                                      ellipseRadius, //width of ellipse
                                      ellipseRadius); //height of ellipse
                                      //ellipseRadius * 2,
                                      //ellipseRadius * 2);
      }

      g2.setStroke(new BasicStroke(1.0f));
      g2.setPaint(getItemPaint(series, item));
      g2.draw(circle);

      // setup for collecting optional entity info...
      EntityCollection entities = null;
      if (info != null) {
        entities = info.getOwner().getEntityCollection();
      }

      // add an entity for the item...
      if (entities != null) {
        String tip = null;
        if (getToolTipGenerator() != null) {
          tip = getToolTipGenerator().generateToolTip(dataset, series, item);
        }
        String url = null;
        if (getURLGenerator() != null) {
          url = getURLGenerator().generateURL(dataset, series, item);
        }
        XYItemEntity entity = new XYItemEntity(circle, dataset, series, item,
                                               tip, url);
        entities.addEntity(entity);
      }

      // do we need to update the crosshair values?
      if (plot.isDomainCrosshairLockedOnData()) {
        if (plot.isRangeCrosshairLockedOnData()) {
          // both axes
          crosshairInfo.updateCrosshairPoint(x, y, transX, transY);
        }
        else {
          // just the horizontal axis...
          crosshairInfo.updateCrosshairX(x);
        }
      }
      else {
        if (plot.isRangeCrosshairLockedOnData()) {
          // just the vertical axis...
          crosshairInfo.updateCrosshairY(y);
        }
      }
    }

  }

  /**
   * Returns a clone of the renderer, but not a good one.
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException  if the renderer cannot be cloned.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone();
  }

}
