package datavisualizer;

import org.jfree.data.Series;
import java.io.Serializable;
import java.util.Collections;
import java.util.List;

/**
 * Represents a collection of XYZ data items.  Modified from JFreeChart.
 * <p>Title: Data Visualizer</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class XYZSeries
    extends Series
    implements Serializable {
  /** Storage for the data items in the series. */
  protected List data;

  /**
   * Constructs a new xy-series that contains no data.
   * @param name  the series name.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public XYZSeries(String name) {
    super(name);
    data = new java.util.ArrayList();
  }

  /**
   * Returns the number of items in the series.
   *
   * @return The item count.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int getItemCount() {
    return data.size();
  }

  /**
   * Returns the list of data items for the series (the list contains {@link XYZDataItem}
   * objects and is unmodifiable).
   *
   * @return The list of data items.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public List getItems() {
    return Collections.unmodifiableList(this.data);
  }

  /**
   * Adds a data item to the series.
   *
   * @param x  the x value.
   * @param y  the y value.
   * @param z  the z value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void add(double x, double y, double z) {
    XYZDataItem item = new XYZDataItem(x, y, z);
    data.add(item);
  }

  /**
   * Adds a data item to the series.
   *
   * @param x  the x value.
   * @param y  the y value.
   * @param z  the z value.  This can be null.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>June 29, 2004: Created in beta version (LEM)
   */
  public void add(Number x, Number y, Number z) {
    XYZDataItem item = new XYZDataItem(x, y, z);
    data.add(item);
  }

  /**
   * Return the data item with the specified index.
   *
   * @param index  The index.
   *
   * @return The data item with the specified index.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public XYZDataItem getDataItem(int index) {
    return (XYZDataItem) data.get(index);
  }

  /**
   * Returns the x-value at the specified index.
   *
   * @param index  The index.
   *
   * @return The x-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Number getXValue(int index) {
    return getDataItem(index).getX();
  }

  /**
   * Returns the y-value at the specified index.
   *
   * @param index  The index.
   *
   * @return The y-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Number getYValue(int index) {
    return getDataItem(index).getY();
  }

  /**
   * Returns the z-value at the specified index.
   *
   * @param index  The index.
   *
   * @return The z-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Number getZValue(int index) {
    return getDataItem(index).getZ();
  }

  /**
   * Sets the z value at the specified index.
   * @param index int Index.
   * @param z Number Z value.
   */
  public void setZValue(int index, Number z) {
    getDataItem(index).setZ(z);
  }

}
