package javawrapper;

import javawrapper.*;
import java.io.*;
import java.util.Vector;
import org.xml.sax.Attributes;

/**
 * This class manages data for disturbance behaviors.  Planting is not included.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>May 4, 2004:  Extended this to interact with the HarvestWindow
 * class (LEM)
 * <br>May 12, 2004:  Added support for parsing harvest data from a parameter
 * file (LEM)
 * <br>May 13, 2004:  Added the disease behavior (LEM)
 * <br>September 9, 2004:  Added storm behavior (LEM)
 * <br>September 29, 2004: Added storm damage applier behavior (LEM)
 * <br>December 7, 2004:  Changed natural disturbance to episodic mortality (LEM)
 * <br>April 4, 2005:  Added more parameters to storm (LEM)
 * <br>May 2, 2005:  Added more parameters to storm (LEM)
 * <br>May 12, 2005:  Added selection harvest (LEM)
 * <br>January 9, 2006:  Added storm killer behavior (LEM)
 * <br>February 17, 2006:  Added the stormtime data member to the storm
 * damage grid (LEM)
 * <br>March 19, 2006:  Added windstorm behavior (LEM)
 * <br>July 26, 2006: Added cyclicity parameters to windstorm (LEM)
 * <br>September 29, 2006:  Added harvest interface behavior (LEM)
 */

public class DisturbanceBehaviors
    extends BehaviorTypeBase {

  /**Partial cut.  Matches the old code's enum value - that's important!.*/
  public static final int PARTIAL_CUT = 1;

  /**Gap cut.  Matches the old code's enum value - that's important!.*/
  public static final int GAP_CUT = 2;

  /**Clear cut.  Matches the old code's enum value - that's important!.*/
  public static final int CLEAR_CUT = 3;

  /**Percentage basal area.  Matches the old code's enum value - that's important!.*/
  public static final int PERCENTAGE_BASAL_AREA = 2;

  /**Absolute basal area.  Matches the old code's enum value - that's important!.*/
  public static final int ABSOLUTE_BASAL_AREA = 3;

  /**Percentage density.  Matches the old code's enum value - that's important!.*/
  public static final int PERCENTAGE_DENSITY = 0;

  /**Absolute density.  Matches the old code's enum value - that's important!.*/
  public static final int ABSOLUTE_DENSITY = 1;

  /**Our list of harvest cuts, held as HarvestData objects*/
  protected Vector mp_oHarvestCuts = new Vector(0);

  /**List of episodic mortality events, held as HarvestData objects (ignoring the cut type)*/
  protected Vector mp_oMortEpisodes = new Vector(0);

  /**Minimum DBH to which storm damage applies for each species*/
  protected ModelVector mp_fMinStormDamageDBH = new ModelVector(
      "Minimum DBH for Storm Damage, in cm",
      "st_minDBH", "st_mdVal", 0, ModelVector.FLOAT);

  /**Storm damage intercept for medium damage (a) for each species*/
  protected ModelVector mp_fStmDmgInterceptMed = new ModelVector(
      "Storm Damage Intercept (a) for Medium Damage",
      "st_stmDmgInterceptMed", "st_sdimVal", 0, ModelVector.FLOAT);

  /**Storm damage intercept for complete damage (a) for each species*/
  protected ModelVector mp_fStmDmgInterceptFull = new ModelVector(
      "Storm Damage Intercept (a) for Heavy Damage",
      "st_stmDmgInterceptFull", "st_sdifVal", 0, ModelVector.FLOAT);

  /**Storm intensity coefficient (b) for each species*/
  protected ModelVector mp_fStmIntensityCoeff = new ModelVector(
      "Storm Intensity Coefficient (b)",
      "st_stmIntensityCoeff", "st_sicVal", 0, ModelVector.FLOAT);

  /**Storm DBH coefficient (d) for each species*/
  protected ModelVector mp_fStmDBHCoeff = new ModelVector(
      "Storm DBH Coefficient (d)",
      "st_stmDBHCoeff", "st_sdcVal", 0, ModelVector.FLOAT);

  /**Storm - Medium damage probability of survival "a" -
   * species-specific - 01/09/06 LEM*/
  protected ModelVector mp_fStmMedDmgSurvProbA = new ModelVector(
     "Storm Medium Damage Survival Prob Intercept (a)",
     "st_stmMedDmgSurvProbA", "st_smdspaVal", 0, ModelVector.FLOAT);

 /**Storm - Medium damage probability of survival "b" -
   * species-specific - 01/09/06 LEM*/
  protected ModelVector mp_fStmMedDmgSurvProbB = new ModelVector(
     "Storm Medium Damage Survival Prob DBH Coeff. (b)",
     "st_stmMedDmgSurvProbB", "st_smdspbVal", 0, ModelVector.FLOAT);

 /**Storm - Heavy damage probability of survival "a" -
   * species-specific - 01/09/06 LEM*/
  protected ModelVector mp_fStmFullDmgSurvProbA = new ModelVector(
     "Storm Heavy Damage Survival Prob Intercept (a)",
     "st_stmFullDmgSurvProbA", "st_sfdspaVal", 0, ModelVector.FLOAT);

  /**Storm - Heavy damage probability of survival "b" -
   * species-specific - 01/09/06 LEM*/
  protected ModelVector mp_fStmFullDmgSurvProbB = new ModelVector(
     "Storm Heavy Damage Survival Prob DBH Coeff. (b)",
     "st_stmFullDmgSurvProbB", "st_sfdspbVal", 0, ModelVector.FLOAT);

  /**Windstorm - Minimum DBH for windstorm mortality - species-specific -
   * 03/19/06 LEM*/
  protected ModelVector mp_fWindstormMinDBH = new ModelVector(
    "Windstorm - Minimum DBH for Windstorm Mortality",
    "ws_minDBH", "ws_mdVal", 0, ModelVector.FLOAT);

  /**Windstorm - Mortality probability intercept "a" -
   * species-specific - 03/19/06 LEM */
  protected ModelVector mp_fWindstormMortInterceptA = new ModelVector(
     "Windstorm - Mortality Intercept (a)",
     "ws_stmInterceptA", "ws_siaVal", 0, ModelVector.FLOAT);

  /**Windstorm - Mortality probability storm intensity coefficient "c" -
   * species-specific - 03/19/06 LEM */
  protected ModelVector mp_fWindstormStmIntensCoeffC = new ModelVector(
     "Windstorm - Storm Intensity Coefficient (c)",
     "ws_stmIntensCoeffC", "ws_sicVal", 0, ModelVector.FLOAT);

  /**Windstorm - DBH exponent "b" - species-specific - 03/19/06 LEM */
  protected ModelVector mp_fWindstormDBHExpB = new ModelVector(
    "Windstorm - DBH Exponent (b)", "ws_stmDBHExpB", "ws_sdebVal",
    0, ModelVector.FLOAT);

  /**Storm - proportion of trees with total damage that die that tip up -
   * 05/01/05 LEM*/
  protected ModelVector mp_fStmPropFullTipUp = new ModelVector(
      "Storm - Prop. Heavy Damage Dead Trees that Tip Up",
      "st_stmPropTipUpFullDmg", "st_sptufdVal", 0, ModelVector.FLOAT);

  /**Storm return interval for 0-0.1 severity storm*/
  protected ModelFloat m_fClass1RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0 - 0.1", "st_s1ReturnInterval");

  /**Storm return interval for 0.1-0.2 severity storm*/
  protected ModelFloat m_fClass2RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0.1 - 0.2",
      "st_s2ReturnInterval");

  /**Storm return interval for 0.2-0.3 severity storm*/
  protected ModelFloat m_fClass3RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0.2 - 0.3",
      "st_s3ReturnInterval");

  /**Storm return interval for 0.3-0.4 severity storm*/
  protected ModelFloat m_fClass4RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0.3 - 0.4",
      "st_s4ReturnInterval");

  /**Storm return interval for 0.4-0.5 severity storm*/
  protected ModelFloat m_fClass5RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0.4 - 0.5",
      "st_s5ReturnInterval");

  /**Storm return interval for 0.5-0.6 severity storm*/
  protected ModelFloat m_fClass6RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0.5 - 0.6",
      "st_s6ReturnInterval");

  /**Storm return interval for 0.6-0.7 severity storm*/
  protected ModelFloat m_fClass7RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0.6 - 0.7",
      "st_s7ReturnInterval");

  /**Storm return interval for 0.7-0.8 severity storm*/
  protected ModelFloat m_fClass8RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0.7 - 0.8",
      "st_s8ReturnInterval");

  /**Storm return interval for 0.8-0.9 severity storm*/
  protected ModelFloat m_fClass9RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0.8 - 0.9",
      "st_s9ReturnInterval");

  /**Storm return interval for 0.9-1.0 severity storm*/
  protected ModelFloat m_fClass10RtrnInt = new ModelFloat(0,
      "Return Interval for Severity Storm Class 0.9 - 1.0",
      "st_s10ReturnInterval");

  /**Windstorm - Storm intensity for 1 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt1Severity = new ModelFloat(0,
      "Windstorm - Severity for 1 Year Return Interval Storm",
      "ws_severityReturnInterval1");

  /**Windstorm - Storm intensity for 5 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt5Severity = new ModelFloat(0,
      "Windstorm - Severity for 5 Year Return Interval Storm",
      "ws_severityReturnInterval5");

  /**Windstorm - Storm intensity for 10 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt10Severity = new ModelFloat(0,
      "Windstorm - Severity for 10 Year Return Interval Storm",
      "ws_severityReturnInterval10");

  /**Windstorm - Storm intensity for 20 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt20Severity = new ModelFloat(0,
      "Windstorm - Severity for 20 Year Return Interval Storm",
      "ws_severityReturnInterval20");

  /**Windstorm - Storm intensity for 40 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt40Severity = new ModelFloat(0,
      "Windstorm - Severity for 40 Year Return Interval Storm",
      "ws_severityReturnInterval40");

  /**Windstorm - Storm intensity for 80 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt80Severity = new ModelFloat(0,
      "Windstorm - Severity for 80 Year Return Interval Storm",
      "ws_severityReturnInterval80");

  /**Windstorm - Storm intensity for 160 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt160Severity = new ModelFloat(0,
      "Windstorm - Severity for 160 Year Return Interval Storm",
      "ws_severityReturnInterval160");

  /**Windstorm - Storm intensity for 320 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt320Severity = new ModelFloat(0,
      "Windstorm - Severity for 320 Year Return Interval Storm",
      "ws_severityReturnInterval320");

  /**Windstorm - Storm intensity for 640 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt640Severity = new ModelFloat(0,
      "Windstorm - Severity for 640 Year Return Interval Storm",
      "ws_severityReturnInterval640");

  /**Windstorm - Storm intensity for 1280 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt1280Severity = new ModelFloat(0,
      "Windstorm - Severity for 1280 Year Return Interval Storm",
      "ws_severityReturnInterval1280");

  /**Windstorm - Storm intensity for 2560 year return interval - LEM 03/19/06*/
  protected ModelFloat m_fWindstormReturnInt2560Severity = new ModelFloat(0,
      "Windstorm - Severity for 2560 Year Return Interval Storm",
      "ws_severityReturnInterval2560");

  /** Windstorm - SST periodicity (Sr) */
  protected ModelFloat m_fWindstormSSTPeriod = new ModelFloat(1,
      "Windstorm - Sea Surface Temperature Cyclicity Period (Years)",
      "ws_stmSSTPeriod");

  /** Windstorm - Sine function d */
  protected ModelFloat m_fWindstormSineD = new ModelFloat(0,
      "Windstorm - Storm Cyclicity Sine Curve d", "ws_stmSineD");

  /** Windstorm - Sine function f */
  protected ModelFloat m_fWindstormSineF = new ModelFloat(1,
      "Windstorm - Storm Cyclicity Sine Curve f", "ws_stmSineF");

  /** Windstorm - Sine function g */
  protected ModelFloat m_fWindstormSineG = new ModelFloat(1,
      "Windstorm - Storm Cyclicity Sine Curve g", "ws_stmSineG");

  /** Windstorm - Trend function slope (m) */
  protected ModelFloat m_fWindstormTrendSlopeM = new ModelFloat(0,
      "Windstorm - Storm Cyclicity Trend Function Slope (m)",
      "ws_stmTrendSlopeM");

  /** Windstorm - Trend function intercept (i) */
  protected ModelFloat m_fWindstormTrendInterceptI = new ModelFloat(1,
      "Windstorm - Storm Cyclicity Trend Function Intercept (i)",
      "ws_stmTrendInterceptI");

  /**Standard deviation for normal and lognormal stochastic storm damage functions*/
  protected ModelFloat m_fStdDev = new ModelFloat(0,
                                                  "Standard Deviation (lognormal or normal)",
                                                  "st_standardDeviation");

  /**Clumping parameter for negative binomial stochastic storm damage functions*/
  protected ModelFloat m_fClump = new ModelFloat(0,
                                                 "Clumping Parameter (neg. binomial)",
                                                 "st_clumpingParameter");

  /**Selection harvest cut range 1 lower DBH - LEM 05/12/05*/
  protected ModelFloat m_fCut1LoDBH = new ModelFloat(0,
      "Selection Harvest Cut Range 1 Lower DBH (cm)", "sha_loDBH");

  /**Selection harvest cut range 1 upper DBH - LEM 05/12/05*/
  protected ModelFloat m_fCut1HiDBH = new ModelFloat(0,
      "Selection Harvest Cut Range 1 Upper DBH (cm)", "sha_hiDBH");

  /**Selection harvest cut range 1 target BA - LEM 05/12/05*/
  protected ModelFloat m_fCut1BA = new ModelFloat(0,
                                                  "Selection Harvest Cut Range 1 Target Basal Area (m2/ha)",
                                                  "sha_target_BA");

  /**Selection harvest cut range 2 lower DBH - LEM 05/12/05*/
  protected ModelFloat m_fCut2LoDBH = new ModelFloat(0,
      "Selection Harvest Cut Range 2 Lower DBH (cm)", "sha_loDBH");

  /**Selection harvest cut range 2 upper DBH - LEM 05/12/05*/
  protected ModelFloat m_fCut2HiDBH = new ModelFloat(0,
      "Selection Harvest Cut Range 2 Upper DBH (cm)", "sha_hiDBH");

  /**Selection harvest cut range 2 target BA - LEM 05/12/05*/
  protected ModelFloat m_fCut2BA = new ModelFloat(0,
                                                  "Selection Harvest Cut Range 2 Target Basal Area (m2/ha)",
                                                  "sha_target_BA");

  /**Selection harvest cut range 3 lower DBH - LEM 05/12/05*/
  protected ModelFloat m_fCut3LoDBH = new ModelFloat(0,
      "Selection Harvest Cut Range 3 Lower DBH (cm)", "sha_loDBH");

  /**Selection harvest cut range 3 upper DBH - LEM 05/12/05*/
  protected ModelFloat m_fCut3HiDBH = new ModelFloat(0,
      "Selection Harvest Cut Range 3 Upper DBH (cm)", "sha_hiDBH");

  /**Selection harvest cut range 3 target BA - LEM 05/12/05*/
  protected ModelFloat m_fCut3BA = new ModelFloat(0,
                                                  "Selection Harvest Cut Range 3 Target Basal Area (m2/ha)",
                                                  "sha_target_BA");

  /**Selection harvest cut range 4 lower DBH - LEM 05/12/05*/
  protected ModelFloat m_fCut4LoDBH = new ModelFloat(0,
      "Selection Harvest Cut Range 4 Lower DBH (cm)", "sha_loDBH");

  /**Selection harvest cut range 4 upper DBH - LEM 05/12/05*/
  protected ModelFloat m_fCut4HiDBH = new ModelFloat(0,
      "Selection Harvest Cut Range 4 Upper DBH (cm)", "sha_hiDBH");

  /**Selection harvest cut range 4 target BA - LEM 05/12/05*/
  protected ModelFloat m_fCut4BA = new ModelFloat(0,
                                                  "Selection Harvest Cut Range 4 Target Basal Area (m2/ha)",
                                                  "sha_target_BA");

  /**Selection harvest initial age - LEM 05/12/05*/
  protected ModelInt m_iInitialAge = new ModelInt(0,
                                                  "Selection Harvest Initial Age",
                                                  "sha_InitialAge");

  /**Windstorm - Timestep to start storms - LEM 03/19/06*/
  protected ModelInt m_iWindstormTimestepToStartStorms = new ModelInt(0,
      "Windstorm - Timestep to Start Storms", "ws_stmTSToStartStorms");

  /**Number of years damaged trees stay damaged*/
  protected ModelInt m_iNumYearsToHeal = new ModelInt(0,
      "Number of Years Damaged Trees Take to Heal",
      "st_numYearsToHeal");

  /**Number of years storm-damaged snags last before being removed -
   * 05/01/05 LEM*/
  protected ModelInt m_iNumSnagYears = new ModelInt(0,
      "Number of Years Storm-Damaged Snags Last",
      "st_numYearsStormSnags");

  /**Storm susceptibility pattern*/
  protected ModelEnum m_iSusceptibility = new ModelEnum(new int[] {0, 1},
                    new String[] {"Mapped", "Uniform"}
                    , "Plot Storm Susceptibility Pattern", "st_susceptibility");

  /**Storm stochasticity pattern*/
  protected ModelEnum m_iStochasticity = new ModelEnum(new int[] {0, 1},
                    new String[] {"Deterministic", "Stochastic"}
                    , "Storm Damage Application", "st_stochasticity");

  /**Probability distribution function*/
  protected ModelEnum m_iProbDistFunc = new ModelEnum(new int[] {0, 1, 2},
                    new String[] {"Lognormal", "Normal", "Negative Binomial"}
                    , "Stochastic Pattern Damage Distribution",
                    "st_probFunction");


  //*********************************
   // Harvest interface parameters
   //********************************
  /** Path and filename of user executable*/
  protected ModelString m_sHarvIntExecutable = new ModelString("", "", "hi_executable");

  /** Path and filename of file for SORTIE to write with harvestable trees */
  protected ModelString m_sHarvIntSORTIEOutFile = new ModelString("", "", "hi_harvestableTreesFile");

  /** Path and filename of file for executable to write with trees to harvest*/
  protected ModelString m_sHarvIntExecHarvestOutFile = new ModelString("", "", "hi_treesToHarvestFile");

  /** Path and filename of file for executable to write with trees to update*/
  protected ModelString m_sHarvIntExecUpdateOutFile = new ModelString("", "", "hi_treesToUpdateFile");

  /** Path and filename of batch parameters*/
  protected ModelString m_sHarvIntBatchParamsFile = new ModelString("", "", "hi_batchParamsFile");

  /** Path and filename of single-run parameters file to write*/
  protected ModelString m_sHarvIntBatchSingleRunParamsFile = new ModelString("", "", "hi_batchSingleRunParamsFile");

  /** Arguments to pass to executable*/
  protected ModelString m_sHarvIntExecArgs = new ModelString("", "", "hi_executableArguments");

  /** List of new tree data members to add */
  protected Vector mp_sHarvIntNewTreeDataMembers = new Vector(0);

  /** List of file columns */
  protected ModelVector mp_sHarvIntFileColumns = new ModelVector("",
      "hi_dataMembers", "hi_dataMember", 0, ModelVector.STRING);

  /** How often harvests occur, in years */
  protected ModelInt m_iHarvIntHarvestPeriod = new ModelInt(1, "", "hi_harvestPeriod");




  /**Placeholder for reading cut ranges from an XML file.  This one is
   * for DBH range lower bound*/
  protected float m_iCutRangeLowerBound = -1;

  /**Placeholder for reading cut ranges from an XML file.  This one is
   * for DBH range upper bound*/
  protected float m_iCutRangeUpperBound = -1;

  /**Placeholder for reading cut ranges from an XML file.  This one is
   * for range cut amount*/
  protected float m_iCutRangeAmount = -1;

  /**Placeholder for reading selection harvest cut ranges from an XML file.*/
  protected int m_iSelectionHarvestCutRange = 0;

  /**
   * Constructor.
   * @param oManager GUIManager object.
   * @throws ModelException if there's a problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DisturbanceBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Disturbance", "");

    //Set up disturbance behaviors
    mp_oChildBehaviors = new Behavior[8];

    mp_oChildBehaviors[0] = new Behavior("Selection Harvest",
                                         "Selection Harvest",
                                         "SelectionHarvest",
                                         (float) 1.0);
    mp_oChildBehaviors[0].m_bMustHaveTrees = false;
    mp_oChildBehaviors[0].AddRequiredData(m_iInitialAge);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut1LoDBH);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut1HiDBH);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut1BA);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut2LoDBH);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut2HiDBH);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut2BA);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut3LoDBH);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut3HiDBH);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut3BA);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut4LoDBH);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut4HiDBH);
    mp_oChildBehaviors[0].AddRequiredData(m_fCut4BA);

    //Harvest behavior
    mp_oChildBehaviors[1] = new Behavior("Harvest",
                                         "Harvest", "harvest", (float) 1);
    mp_oChildBehaviors[1].m_bCanEditAutomatically = false;
    mp_oChildBehaviors[1].m_bMustHaveTrees = false;

    //Episodic mortality behavior
    mp_oChildBehaviors[2] = new Behavior("Episodic Mortality",
                                         "Episodic Mortality",
                                         "episodic mortality",
                                         (float) 1);
    mp_oChildBehaviors[2].m_bCanEditAutomatically = false;
    mp_oChildBehaviors[2].m_bMustHaveTrees = false;

    //Storm behavior
    mp_oChildBehaviors[3] = new Behavior("Storms", "Storms", "storm", (float) 1);
    mp_oChildBehaviors[3].m_bMustHaveTrees = false;
    mp_oChildBehaviors[3].AddRequiredData(m_fClass1RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_fClass2RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_fClass3RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_fClass4RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_fClass5RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_fClass6RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_fClass7RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_fClass8RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_fClass9RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_fClass10RtrnInt);
    mp_oChildBehaviors[3].AddRequiredData(m_iSusceptibility);
    mp_oChildBehaviors[3].AddRequiredData(m_iStochasticity);
    mp_oChildBehaviors[3].AddRequiredData(m_iProbDistFunc);
    mp_oChildBehaviors[3].AddRequiredData(m_fStdDev);
    mp_oChildBehaviors[3].AddRequiredData(m_fClump);

    //Storm Damage Applier behavior
    mp_oChildBehaviors[4] = new Behavior("Storm Damage Applier",
                                         "Storm Damage Applier",
                                         "storm damage applier",
                                         (float) 1.0);
    mp_oChildBehaviors[4].m_bMustHaveTrees = true;
    mp_oChildBehaviors[4].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[4].AddRequiredData(mp_fMinStormDamageDBH);
    mp_oChildBehaviors[4].AddRequiredData(mp_fStmDmgInterceptMed);
    mp_oChildBehaviors[4].AddRequiredData(mp_fStmDmgInterceptFull);
    mp_oChildBehaviors[4].AddRequiredData(mp_fStmIntensityCoeff);
    mp_oChildBehaviors[4].AddRequiredData(mp_fStmDBHCoeff);
    mp_oChildBehaviors[4].AddRequiredData(m_iNumYearsToHeal);
    //Storm damage counter data member
    mp_oChildBehaviors[4].mp_oNewTreeDataMembers.add(new DataMember(
        "Storm Damage Value", "stm_dmg", DataMember.INTEGER));

    //Storm Killer behavior
    mp_oChildBehaviors[5] = new Behavior("Storm Killer", "Storm Killer",
                                         "storm killer", (float) 1.0);
    mp_oChildBehaviors[5].m_bMustHaveTrees = true;
    mp_oChildBehaviors[5].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[5].AddRequiredData(mp_fMinStormDamageDBH);
    mp_oChildBehaviors[5].AddRequiredData(mp_fStmMedDmgSurvProbA);
    mp_oChildBehaviors[5].AddRequiredData(mp_fStmMedDmgSurvProbB);
    mp_oChildBehaviors[5].AddRequiredData(mp_fStmFullDmgSurvProbA);
    mp_oChildBehaviors[5].AddRequiredData(mp_fStmFullDmgSurvProbB);
    mp_oChildBehaviors[5].AddRequiredData(mp_fStmPropFullTipUp);
    mp_oChildBehaviors[5].AddRequiredData(m_iNumSnagYears);

    //Windstorm behavior
    mp_oChildBehaviors[6] = new Behavior("Windstorm", "Windstorm",
                                         "Windstorm", (float) 2.0);
    mp_oChildBehaviors[6].m_bMustHaveTrees = true;
    mp_oChildBehaviors[6].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[6].AddRequiredData(mp_fWindstormMortInterceptA);
    mp_oChildBehaviors[6].AddRequiredData(mp_fWindstormStmIntensCoeffC);
    mp_oChildBehaviors[6].AddRequiredData(mp_fWindstormDBHExpB);
    mp_oChildBehaviors[6].AddRequiredData(mp_fWindstormMinDBH);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt1Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt5Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt10Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt20Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt40Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt80Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt160Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt320Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt640Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt1280Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormReturnInt2560Severity);
    mp_oChildBehaviors[6].AddRequiredData(m_iWindstormTimestepToStartStorms);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormSSTPeriod);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormSineD);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormSineF);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormSineG);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormTrendSlopeM);
    mp_oChildBehaviors[6].AddRequiredData(m_fWindstormTrendInterceptI);

    //Harvest interface
    mp_oChildBehaviors[7] = new Behavior("Harvest Interface",
                                         "Harvest Interface", "Harvest Interface", (float) 1);
    mp_oChildBehaviors[7].m_bCanEditAutomatically = false;
    mp_oChildBehaviors[7].m_bMustHaveTrees = false;
/*    mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntExecutable);
    mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntSORTIEOutFile);
    mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntExecHarvestOutFile);
    mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntExecUpdateOutFile);
    mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntBatchParamsFile);
    mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntBatchSingleRunParamsFile);
    mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntExecArgs);
    mp_oChildBehaviors[7].AddRequiredData(mp_sHarvIntFileColumns);
    mp_oChildBehaviors[7].AddRequiredData(m_iHarvIntHarvestPeriod);*/


    mp_oAllData.add(m_fClass1RtrnInt);
    mp_oAllData.add(m_fClass2RtrnInt);
    mp_oAllData.add(m_fClass3RtrnInt);
    mp_oAllData.add(m_fClass4RtrnInt);
    mp_oAllData.add(m_fClass5RtrnInt);
    mp_oAllData.add(m_fClass6RtrnInt);
    mp_oAllData.add(m_fClass7RtrnInt);
    mp_oAllData.add(m_fClass8RtrnInt);
    mp_oAllData.add(m_fClass9RtrnInt);
    mp_oAllData.add(m_fClass10RtrnInt);
    mp_oAllData.add(m_iSusceptibility);
    mp_oAllData.add(m_iStochasticity);
    mp_oAllData.add(m_iProbDistFunc);
    mp_oAllData.add(m_fStdDev);
    mp_oAllData.add(m_fClump);
    mp_oAllData.add(mp_fMinStormDamageDBH);
    mp_oAllData.add(mp_fStmDmgInterceptMed);
    mp_oAllData.add(mp_fStmDmgInterceptFull);
    mp_oAllData.add(mp_fStmIntensityCoeff);
    mp_oAllData.add(mp_fStmDBHCoeff);
    mp_oAllData.add(mp_fStmMedDmgSurvProbA);
    mp_oAllData.add(mp_fStmMedDmgSurvProbB);
    mp_oAllData.add(mp_fStmFullDmgSurvProbA);
    mp_oAllData.add(mp_fStmFullDmgSurvProbB);
    mp_oAllData.add(m_iNumYearsToHeal);
    mp_oAllData.add(mp_fStmPropFullTipUp);
    mp_oAllData.add(m_iNumSnagYears);
    mp_oAllData.add(m_iInitialAge);
    mp_oAllData.add(m_fCut1LoDBH);
    mp_oAllData.add(m_fCut1HiDBH);
    mp_oAllData.add(m_fCut1BA);
    mp_oAllData.add(m_fCut2LoDBH);
    mp_oAllData.add(m_fCut2HiDBH);
    mp_oAllData.add(m_fCut2BA);
    mp_oAllData.add(m_fCut3LoDBH);
    mp_oAllData.add(m_fCut3HiDBH);
    mp_oAllData.add(m_fCut3BA);
    mp_oAllData.add(m_fCut4LoDBH);
    mp_oAllData.add(m_fCut4HiDBH);
    mp_oAllData.add(m_fCut4BA);
    mp_oAllData.add(mp_fWindstormMortInterceptA);
    mp_oAllData.add(mp_fWindstormStmIntensCoeffC);
    mp_oAllData.add(mp_fWindstormDBHExpB);
    mp_oAllData.add(mp_fWindstormMinDBH);
    mp_oAllData.add(m_fWindstormReturnInt1Severity);
    mp_oAllData.add(m_fWindstormReturnInt5Severity);
    mp_oAllData.add(m_fWindstormReturnInt10Severity);
    mp_oAllData.add(m_fWindstormReturnInt20Severity);
    mp_oAllData.add(m_fWindstormReturnInt40Severity);
    mp_oAllData.add(m_fWindstormReturnInt80Severity);
    mp_oAllData.add(m_fWindstormReturnInt160Severity);
    mp_oAllData.add(m_fWindstormReturnInt320Severity);
    mp_oAllData.add(m_fWindstormReturnInt640Severity);
    mp_oAllData.add(m_fWindstormReturnInt1280Severity);
    mp_oAllData.add(m_fWindstormReturnInt2560Severity);
    mp_oAllData.add(m_iWindstormTimestepToStartStorms);
    mp_oAllData.add(m_fWindstormSSTPeriod);
    mp_oAllData.add(m_fWindstormSineD);
    mp_oAllData.add(m_fWindstormSineF);
    mp_oAllData.add(m_fWindstormSineG);
    mp_oAllData.add(m_fWindstormTrendSlopeM);
    mp_oAllData.add(m_fWindstormTrendInterceptI);
    mp_oAllData.add(m_sHarvIntExecutable);
    mp_oAllData.add(m_sHarvIntSORTIEOutFile);
    mp_oAllData.add(m_sHarvIntExecHarvestOutFile);
    mp_oAllData.add(m_sHarvIntExecUpdateOutFile);
    mp_oAllData.add(m_sHarvIntBatchParamsFile);
    mp_oAllData.add(m_sHarvIntBatchSingleRunParamsFile);
    mp_oAllData.add(m_sHarvIntExecArgs);
    mp_oAllData.add(mp_sHarvIntFileColumns);
    mp_oAllData.add(m_iHarvIntHarvestPeriod);

    //We have to wait to set up the grids
  }

  /**
   * Reads in an old tab-delimited harvest regime file.  Any existing harvest
   * data will be replaced with what's in the file.
   * @param sFileName Filename of harvest regime file.
   * @param oPlot Plot object.
   * @param oPop TreePopulation object.
   * @throws ModelException Wrapping an IO exception, or if the harvest
   * parameters do not match the currently loaded parameter file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void ReadOldHarvestRegimeFile(String sFileName, Plot oPlot,
                                       TreePopulation oPop) throws
      ModelException {
    FileReader oIn = null;
    try {
      oIn = new FileReader(sFileName);
      Cell[] p_oCells;
      HarvestData oHarvest = null;
      Vector oLine, oHarvestList = new Vector(0);
      String sVal;
      Integer iVal;
      float fLowDbh, fHighDbh, fAmountToCut,
          fOldXCellLength, fOldYCellLength; //old X and Y cell lengths
      int iNumOldGridCells,
          iOldNumXGrids, iOldNumYGrids, //old number of X and Y grids
          iNumSpecies = oPop.GetNumberOfSpecies(),
          iTimestep,
          iSp, iGrid, i;

      ModelFileFunctions.SkipLine(oIn);

      //Get the first data line
      oLine = ModelFileFunctions.ReadLine(oIn);

      //Validate the plot lengths - remember they're reversed from the old to the
      //new coordinate system
      sVal = (String) oLine.elementAt(2); //old X cell length
      fOldXCellLength = new Float(sVal).floatValue();
      sVal = (String) oLine.elementAt(0); //old number of X cells
      iOldNumXGrids = new Integer(sVal).intValue();

      sVal = (String) oLine.elementAt(3); //old Y plot length
      fOldYCellLength = new Float(sVal).floatValue();
      sVal = (String) oLine.elementAt(1); //old number of Y cells
      iOldNumYGrids = new Integer(sVal).intValue();

      if (fOldXCellLength * (float) iOldNumXGrids != oPlot.GetPlotYLength() ||
          fOldYCellLength * (float) iOldNumYGrids != oPlot.GetPlotXLength()) {
        ModelException oErr = new ModelException();
        oErr.sCallingFunction = "JAVA";
        oErr.iErrorCode = ErrorGUI.BAD_FILE;
        oErr.sMessage = "The plot dimensions for this harvest regime file " +
            "do not match the plot dimensions for this parameter file.";
        throw oErr;
      }

      //Validate the number of species
      sVal = (String) oLine.elementAt(4);
      if (new Integer(sVal).intValue() != iNumSpecies) {
        ModelException oErr = new ModelException();
        oErr.sCallingFunction = "JAVA";
        oErr.iErrorCode = ErrorGUI.BAD_FILE;
        oErr.sMessage = "The number of species for this harvest regime file (" +
            sVal +
            ") does not match the plot length for this parameter file (" +
            String.valueOf(iNumSpecies) + ".";
        throw oErr;
      }

      //Check the what's to load flag - if planting only, skip out
      sVal = (String) oLine.elementAt(5);
      if (new Integer(sVal).intValue() == 2) { //2 = planting only
        return;
      }

      //Calculate the number of grid cells in this file
      sVal = (String) oLine.elementAt(0); //number of X cells
      iNumOldGridCells = new Integer(sVal).intValue();
      sVal = (String) oLine.elementAt(1); //number of Y cells
      iNumOldGridCells *= new Integer(sVal).intValue();

      for (iGrid = 0; iGrid < iNumOldGridCells; iGrid++) {
        for (iSp = 0; iSp < iNumSpecies; iSp++) {

          //Then create event records and read in the data
          oLine = ModelFileFunctions.ReadLine(oIn);

          //When this record is over, it will be designated by a -1 in the
          //first bucket of the line.  Read all the info for this grid and
          //species until we get to that end-of-record designator.
          sVal = (String) oLine.elementAt(0);
          iVal = new Integer(sVal);
          while (iVal.intValue() > -1) {

            //If there are only two values on the line - no values - skip out
            if (oLine.size() > 2) {
              //First value is grid cell - make sure it matches our current grid
              if (iVal.intValue() != iGrid) {
                ModelException oErr = new ModelException();
                oErr.iErrorCode = ErrorGUI.TAB_FILE_PREMATURE_END;
                oErr.sCallingFunction = "ReadOldHarvestRegimeFile";
                oErr.sMessage = sFileName;
              }

              //Second value is species number - make sure it matches our current
              //species
              sVal = (String) oLine.elementAt(1);
              if (new Integer(sVal).intValue() != iSp) {
                ModelException oErr = new ModelException();
                oErr.iErrorCode = ErrorGUI.TAB_FILE_PREMATURE_END;
                oErr.sCallingFunction = "ReadOldHarvestRegimeFile";
                oErr.sMessage = sFileName;
              }

              //Third value is timestep
              sVal = (String) oLine.elementAt(2);
              iTimestep = new Integer(sVal).intValue();

              //Create a new harvest data record
              oHarvest = null;
              oHarvest = new HarvestData();

              //Set timestep
              oHarvest.SetTimestep(iTimestep);

              //Set species
              oHarvest.AddSpecies(iSp);

              //Add cut amount type
              sVal = (String) oLine.elementAt(11);
              oHarvest.SetCutAmountType(new Integer(sVal).intValue());

              //Add cut type
              sVal = (String) oLine.elementAt(16);
              oHarvest.SetCutType(new Integer(sVal).intValue());

              //Add grid cells
              p_oCells = Cell.TransformOldGridCell(iGrid, fOldXCellLength,
                  fOldYCellLength,
                  iOldNumXGrids, iOldNumYGrids, oPlot);
              for (i = 0; i < p_oCells.length; i++) {
                oHarvest.AddCell(p_oCells[i]);
              }

              //Add cut ranges - there are four
              sVal = (String) oLine.elementAt(3); //first range low dbh
              fLowDbh = new Float(sVal).floatValue();
              sVal = (String) oLine.elementAt(7); //first range high dbh
              fHighDbh = new Float(sVal).floatValue();
              sVal = (String) oLine.elementAt(12); //first range low cut amount
              fAmountToCut = new Float(sVal).floatValue();
              if (fAmountToCut > 0) {
                oHarvest.AddCutRange(fLowDbh, fHighDbh, fAmountToCut);
              }

              sVal = (String) oLine.elementAt(4); //second range low dbh
              fLowDbh = new Float(sVal).floatValue();
              sVal = (String) oLine.elementAt(8); //second range high dbh
              fHighDbh = new Float(sVal).floatValue();
              sVal = (String) oLine.elementAt(13); //second range low cut amount
              fAmountToCut = new Float(sVal).floatValue();
              if (fAmountToCut > 0) {
                oHarvest.AddCutRange(fLowDbh, fHighDbh, fAmountToCut);
              }

              sVal = (String) oLine.elementAt(5); //third range low dbh
              fLowDbh = new Float(sVal).floatValue();
              sVal = (String) oLine.elementAt(9); //third range high dbh
              fHighDbh = new Float(sVal).floatValue();
              sVal = (String) oLine.elementAt(14); //third range low cut amount
              fAmountToCut = new Float(sVal).floatValue();
              if (fAmountToCut > 0) {
                oHarvest.AddCutRange(fLowDbh, fHighDbh, fAmountToCut);
              }

              sVal = (String) oLine.elementAt(6); //fourth range low dbh
              fLowDbh = new Float(sVal).floatValue();
              sVal = (String) oLine.elementAt(10); //fourth range high dbh
              fHighDbh = new Float(sVal).floatValue();
              sVal = (String) oLine.elementAt(15); //fourth range low cut amount
              fAmountToCut = new Float(sVal).floatValue();
              if (fAmountToCut > 0) {
                oHarvest.AddCutRange(fLowDbh, fHighDbh, fAmountToCut);
              }

              //Add to our list
              oHarvestList = AddHarvestData(oHarvestList, oHarvest);

            }

            oLine = ModelFileFunctions.ReadLine(oIn);
            sVal = (String) oLine.elementAt(0);
            iVal = new Integer(sVal);
          }
        }
      }

      //Now compact our list and copy it to our existing list
      mp_oHarvestCuts = CompactHarvestList(oHarvestList);

      Behavior oBeh = GetBehaviorByKey("Harvest");
      if (mp_oHarvestCuts.size() > 0) {
        oBeh.m_bIsEnabled = true;
      }
      else {
        oBeh.m_bIsEnabled = false;
      }
    }
    catch (java.io.IOException oErr) {
      ModelException oExp = new ModelException();
      oExp.iErrorCode = ErrorGUI.BAD_FILE;
      oExp.sCallingFunction = "ReadOldHarvestRegimeFile";
      oExp.sMessage = sFileName;
      throw (oExp);
    }
    finally {
      try {
        oIn.close();
      }
      catch (java.io.IOException e) {
        ;
      }
    }
  }

  /**
   * This adds the data contained in a HarvestData object to a list of harvest
   * data.  A HarvestData object is first checked to make sure there are any
   * cut ranges defined; if not, nothing happens.  If there is data to be
   * added, it is compared to existing HarvestData objects on the list.  If
   * they match except for the grid cell, then the new object's grid cell is
   * added to the existing object and the new object thrown away.  If it is
   * truly new data, it is added to the list.
   *
   * IMPORTANT:  It is assumed that the harvest data list is all single-species
   * cuts.  This makes this method not very generalized.
   * @param oList The list of HarvestData objects to add to.
   * @param oNewHarvest The HarvestData object to add.
   * @return The new list with the data added.
   * @throws ModelException Passing through underlying exceptions.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected Vector AddHarvestData(Vector oList, HarvestData oNewHarvest) throws
      ModelException {
    HarvestData oExistingHarvest;
    int i, j, k;
    boolean bMatch, bAllMatch;

    if (null == oNewHarvest) {
      return oList;
    }

    //If there are no cut ranges, don't add
    if (oNewHarvest.GetNumberOfCutRanges() == 0) {
      return null;
    }

    //If the list is currently null, create it
    if (null == oList) {
      oList = new Vector(0);
      oList.add(oNewHarvest);
      return oList;
    }

    for (i = 0; i < oList.size(); i++) {

      oExistingHarvest = (HarvestData) oList.elementAt(i);

      //Compare all the data
      if (oExistingHarvest.GetCutAmountType() == oNewHarvest.GetCutAmountType() &&
          oExistingHarvest.GetCutType() == oNewHarvest.GetCutType() &&
          oExistingHarvest.GetTimestep() == oNewHarvest.GetTimestep() &&
          oExistingHarvest.GetNumberOfCutRanges() ==
          oNewHarvest.GetNumberOfCutRanges() &&
          oExistingHarvest.GetSpecies(0) == oNewHarvest.GetSpecies(0)) {

        //Make sure the cut ranges are the same
        bAllMatch = true;
        for (j = 0; j < oNewHarvest.GetNumberOfCutRanges(); j++) {
          bMatch = false;
          for (k = 0; k < oExistingHarvest.GetNumberOfCutRanges(); k++) {
            if (oNewHarvest.GetLowerBound(j) ==
                oExistingHarvest.GetLowerBound(k) &&
                oNewHarvest.GetUpperBound(j) ==
                oExistingHarvest.GetUpperBound(k) &&
                oNewHarvest.GetCutAmount(j) == oExistingHarvest.GetCutAmount(k)) {
              bMatch = true;
              break;
            }
          }
          if (!bMatch) {
            bAllMatch = false;
            break;
          }
        }

        if (bAllMatch) {
          //Add this one's grid cell to the existing list
          for (j = 0; j < oNewHarvest.GetNumberOfCells(); j++) {
            oExistingHarvest.AddCell(oNewHarvest.GetCell(j));
          }
          return oList;
        }
      }
    }

    //If we've gotten to here, then the new object's data is unique and should
    //be added.
    oList.add(oNewHarvest);
    return oList;
  }

  /**
   * Takes a list of HarvestData objects where each is one single species, and
   * replaces it with a list of HarvestData objects for multiple species for
   * any whose data match.
   * @param oList The list to compact.
   * @return The compacted list.
   * @throws ModelException Passing through underlying exceptions
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected Vector CompactHarvestList(Vector oList) throws ModelException {

    Vector oTempList = new Vector(0);
    HarvestData oExistingHarvest, oNewHarvest;
    Cell oNewCell, oExistingCell;
    int i, j, k, m;
    boolean bUnique = true, bMatch, bAllMatch;

    //If the list is null, return
    if (null == oList) {
      return null;
    }

    for (i = 0; i < oList.size(); i++) {

      oNewHarvest = (HarvestData) oList.elementAt(i);
      bUnique = true;
      //See if this object's data matches any existing object
      for (j = 0; j < oTempList.size(); j++) {

        oExistingHarvest = (HarvestData) oTempList.elementAt(j);

        if (oExistingHarvest.GetCutAmountType() == oNewHarvest.GetCutAmountType() &&
            oExistingHarvest.GetCutType() == oNewHarvest.GetCutType() &&
            oExistingHarvest.GetTimestep() == oNewHarvest.GetTimestep() &&
            oExistingHarvest.GetNumberOfCutRanges() ==
            oNewHarvest.GetNumberOfCutRanges() &&
            oExistingHarvest.GetNumberOfCells() == oNewHarvest.GetNumberOfCells()) {

          //Test cut ranges
          bAllMatch = true;
          for (k = 0; k < oNewHarvest.GetNumberOfCutRanges(); k++) {
            bMatch = false;
            for (m = 0; m < oExistingHarvest.GetNumberOfCutRanges(); m++) {
              if (oNewHarvest.GetLowerBound(k) ==
                  oExistingHarvest.GetLowerBound(m) &&
                  oNewHarvest.GetUpperBound(k) ==
                  oExistingHarvest.GetUpperBound(m) &&
                  oNewHarvest.GetCutAmount(k) ==
                  oExistingHarvest.GetCutAmount(m)) {
                bMatch = true;
                break;
              }
            }
            if (!bMatch) {
              bAllMatch = false;
              break;
            }
          }

          //If cut ranges checked out, test the cell list
          if (bAllMatch) {

            bAllMatch = true;
            for (k = 0; k < oNewHarvest.GetNumberOfCells(); k++) {
              oNewCell = (Cell) oNewHarvest.GetCell(k);
              bMatch = false;
              for (m = 0; m < oExistingHarvest.GetNumberOfCells(); m++) {
                oExistingCell = (Cell) oExistingHarvest.GetCell(m);
                if (oNewCell.GetX() == oExistingCell.GetX() &&
                    oNewCell.GetY() == oExistingCell.GetY()) {
                  bMatch = true;
                  break;
                }
              }
              if (!bMatch) {
                bAllMatch = false;
                break;
              }
            }
            if (bAllMatch) {
              bUnique = false;
              oExistingHarvest.AddSpecies(oNewHarvest.GetSpecies(0));
            }
          }
        }
      }

      if (bUnique) {
        //This object has unique data - add it to the temp list
        oTempList.add(oNewHarvest);
      }
    }

    //Return the temp list to the existing list
    return oTempList;
  }

  /**
   * Writes the storm data to the parameter file.  Does nothing if the
   * storm behaviors are not enabled.
   * @param jOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   * @throws ModelException passed through from called functions.
   */
  private void WriteStorm(BufferedWriter jOut) throws java.io.IOException,
      ModelException {

    if (GetBehaviorByKey("Storms").m_bIsEnabled) {
      jOut.write("<storm>");

      WriteData(jOut, m_fClass1RtrnInt);
      WriteData(jOut, m_fClass2RtrnInt);
      WriteData(jOut, m_fClass3RtrnInt);
      WriteData(jOut, m_fClass4RtrnInt);
      WriteData(jOut, m_fClass5RtrnInt);
      WriteData(jOut, m_fClass6RtrnInt);
      WriteData(jOut, m_fClass7RtrnInt);
      WriteData(jOut, m_fClass8RtrnInt);
      WriteData(jOut, m_fClass9RtrnInt);
      WriteData(jOut, m_fClass10RtrnInt);
      WriteData(jOut, m_iSusceptibility);
      WriteData(jOut, m_iStochasticity);
      if (m_iStochasticity.GetStringValue().equals("Stochastic")) {
        WriteData(jOut, m_iProbDistFunc);

        if (m_iProbDistFunc.GetStringValue().equalsIgnoreCase("Lognormal") ||
            m_iProbDistFunc.GetStringValue().equalsIgnoreCase("Normal")) {
          WriteData(jOut, m_fStdDev);
        }
        else if (m_iProbDistFunc.GetStringValue().equalsIgnoreCase(
            "Negative Binomial")) {
          WriteData(jOut, m_fClump);
        }
      }

      if (GetBehaviorByKey("Storm Damage Applier").m_bIsEnabled) {
        TreePopulation oPop = m_oManager.GetTreePopulation();
        boolean[] p_bAppliesTo = GetBehaviorByKey("Storm Damage Applier").
            GetWhichSpeciesUsed(oPop);
        WriteSpeciesSpecificValue(jOut, mp_fMinStormDamageDBH, oPop, p_bAppliesTo);
        WriteSpeciesSpecificValue(jOut, mp_fStmDmgInterceptMed, oPop,
                                  p_bAppliesTo);
        WriteSpeciesSpecificValue(jOut, mp_fStmDmgInterceptFull, oPop,
                                  p_bAppliesTo);
        WriteSpeciesSpecificValue(jOut, mp_fStmIntensityCoeff, oPop,
                                  p_bAppliesTo);
        WriteSpeciesSpecificValue(jOut, mp_fStmDBHCoeff, oPop, p_bAppliesTo);
        WriteData(jOut, m_iNumYearsToHeal);
      }

      if (GetBehaviorByKey("Storm Killer").m_bIsEnabled) {
        TreePopulation oPop = m_oManager.GetTreePopulation();
        boolean[] p_bAppliesTo = GetBehaviorByKey("Storm Killer").
            GetWhichSpeciesUsed(oPop);
        WriteSpeciesSpecificValue(jOut, mp_fStmMedDmgSurvProbA, oPop,
                                  p_bAppliesTo);
        WriteSpeciesSpecificValue(jOut, mp_fStmMedDmgSurvProbB, oPop, p_bAppliesTo);
        WriteSpeciesSpecificValue(jOut, mp_fStmFullDmgSurvProbA, oPop,
                                  p_bAppliesTo);
        WriteSpeciesSpecificValue(jOut, mp_fStmFullDmgSurvProbB, oPop,
                                  p_bAppliesTo);
        WriteSpeciesSpecificValue(jOut, mp_fStmPropFullTipUp, oPop,
                                  p_bAppliesTo);
        WriteData(jOut, m_iNumSnagYears);
      }

      jOut.write("</storm>");
    }
  }

  /**
   * Writes the windstorm data to the parameter file.  Does nothing if the
   * windstorm behavior is not enabled.
   * @param jOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   * @throws ModelException passed through from called functions.
   */
  private void WriteWindstorm(BufferedWriter jOut) throws java.io.IOException,
      ModelException {

    Behavior oBeh = GetBehaviorByKey("Windstorm");
    if (oBeh.m_bIsEnabled) {

      TreePopulation oPop = m_oManager.GetTreePopulation();
      boolean[] p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);

      jOut.write("<windstorm>");

      WriteData(jOut, m_fWindstormReturnInt1Severity);
      WriteData(jOut, m_fWindstormReturnInt5Severity);
      WriteData(jOut, m_fWindstormReturnInt10Severity);
      WriteData(jOut, m_fWindstormReturnInt20Severity);
      WriteData(jOut, m_fWindstormReturnInt40Severity);
      WriteData(jOut, m_fWindstormReturnInt80Severity);
      WriteData(jOut, m_fWindstormReturnInt160Severity);
      WriteData(jOut, m_fWindstormReturnInt320Severity);
      WriteData(jOut, m_fWindstormReturnInt640Severity);
      WriteData(jOut, m_fWindstormReturnInt1280Severity);
      WriteData(jOut, m_fWindstormReturnInt2560Severity);
      WriteData(jOut, m_fWindstormSSTPeriod);
      WriteData(jOut, m_fWindstormSineD);
      WriteData(jOut, m_fWindstormSineF);
      WriteData(jOut, m_fWindstormSineG);
      WriteData(jOut, m_fWindstormTrendSlopeM);
      WriteData(jOut, m_fWindstormTrendInterceptI);
      WriteSpeciesSpecificValue(jOut, mp_fWindstormMinDBH, oPop, p_bAppliesTo);
      WriteSpeciesSpecificValue(jOut, mp_fWindstormMortInterceptA, oPop, p_bAppliesTo);
      WriteSpeciesSpecificValue(jOut, mp_fWindstormStmIntensCoeffC, oPop, p_bAppliesTo);
      WriteSpeciesSpecificValue(jOut, mp_fWindstormDBHExpB, oPop, p_bAppliesTo);

      WriteData(jOut, m_iWindstormTimestepToStartStorms);

      jOut.write("</windstorm>");
    }
  }

  /**
   * Writes the selection harvest data to the parameter file.  Does nothing if
   * the selection harvest behavior is not enabled.
   * @param jOut FileWriter File to write to.
   * @throws IOException if there is a problem writing the file.
   * @throws ModelException passed through from called functions.
   */
  private void WriteSelectionHarvest(BufferedWriter jOut) throws java.io.
      IOException,
      ModelException {

    if (GetBehaviorByKey("Selection Harvest").m_bIsEnabled) {
      jOut.write("<SelectionHarvest>");

      WriteData(jOut, m_iInitialAge);

      if (m_fCut1BA.GetValue() > 0) {
        jOut.write("<sha_CutRange>");
        WriteData(jOut, m_fCut1LoDBH);
        WriteData(jOut, m_fCut1HiDBH);
        WriteData(jOut, m_fCut1BA);
        jOut.write("</sha_CutRange>");
      }
      if (m_fCut2BA.GetValue() > 0) {
        jOut.write("<sha_CutRange>");
        WriteData(jOut, m_fCut2LoDBH);
        WriteData(jOut, m_fCut2HiDBH);
        WriteData(jOut, m_fCut2BA);
        jOut.write("</sha_CutRange>");
      }
      if (m_fCut3BA.GetValue() > 0) {
        jOut.write("<sha_CutRange>");
        WriteData(jOut, m_fCut3LoDBH);
        WriteData(jOut, m_fCut3HiDBH);
        WriteData(jOut, m_fCut3BA);
        jOut.write("</sha_CutRange>");
      }
      if (m_fCut4BA.GetValue() > 0) {
        jOut.write("<sha_CutRange>");
        WriteData(jOut, m_fCut4LoDBH);
        WriteData(jOut, m_fCut4HiDBH);
        WriteData(jOut, m_fCut4BA);
        jOut.write("</sha_CutRange>");
      }

      jOut.write("</SelectionHarvest>");
    }
  }

  /**
   * Writes the episodic mortality data for the parameter file.  Does nothing
   * if there is no episodic mortality data.
   * @param jOut FileWriter File to write to.
   * @param oPop Tree population object.
   * @throws java.io.IOException if there is a problem writing the file
   * @throws ModelException if there are unrecognized disturbance cut types
   * (should never happen).
   */
  private void WriteMortalityEpisode(BufferedWriter jOut, TreePopulation oPop) throws
      java.io.IOException, ModelException {
    HarvestData oData;
    Cell oCell;
    int i, iNumObjects = mp_oMortEpisodes.size(), j, iTemp;

    if (iNumObjects > 0 && GetBehaviorByKey("Episodic Mortality").m_bIsEnabled) {

      jOut.write("<disturbance>");

      //Write each harvest object's data
      for (i = 0; i < iNumObjects; i++) {
        oData = (HarvestData) mp_oMortEpisodes.elementAt(i);

        jOut.write("<ds_deathEvent>");

        //Write each species
        iTemp = oData.GetNumberOfSpecies();
        for (j = 0; j < iTemp; j++) {
          jOut.write("<ds_applyToSpecies species=\"" +
                     oPop.GetSpeciesNameFromCode(oData.GetSpecies(j)) +
                     "\"/>");
        }

        //Write the timestep
        jOut.write("<ds_timestep>" + String.valueOf(oData.GetTimestep()) +
                   "</ds_timestep>");

        //Write the cut amount type
        jOut.write("<ds_cutAmountType>");
        iTemp = oData.GetCutAmountType();
        if (iTemp == ABSOLUTE_BASAL_AREA) {
          jOut.write("absolute basal area");
        }
        else if (iTemp == ABSOLUTE_DENSITY) {
          jOut.write("absolute density");
        }
        else if (iTemp == PERCENTAGE_BASAL_AREA) {
          jOut.write("percent of basal area");
        }
        else if (iTemp == PERCENTAGE_DENSITY) {
          jOut.write("percent of density");
        }
        else {
          ModelException oErr = new ModelException();
          oErr.iErrorCode = ErrorGUI.BAD_DATA;
          oErr.sCallingFunction = "JAVA";
          oErr.sMessage =
              "When writing parameter file, came across unrecognized " +
              "mortality episode cut type \"" + String.valueOf(iTemp) + "\".";
          throw oErr;
        }
        jOut.write("</ds_cutAmountType>");

        //Write the cut ranges
        iTemp = oData.GetNumberOfCutRanges();
        for (j = 0; j < iTemp; j++) {

          jOut.write("<ds_dbhRange>");
          jOut.write("<ds_low>" + String.valueOf(oData.GetLowerBound(j)) +
                     "</ds_low>");
          jOut.write("<ds_high>" + String.valueOf(oData.GetUpperBound(j)) +
                     "</ds_high>");
          jOut.write("<ds_amountToCut>" +
                     String.valueOf(oData.GetCutAmount(j)) +
                     "</ds_amountToCut>");
          jOut.write("</ds_dbhRange>");
        }

        //Write the list of cells
        iTemp = oData.GetNumberOfCells();
        for (j = 0; j < iTemp; j++) {

          oCell = (Cell) oData.GetCell(j);

          jOut.write("<ds_applyToCell x=\"" + String.valueOf(oCell.GetX())
                     + "\" y=\"" + String.valueOf(oCell.GetY()) + "\"/>");
        }

        jOut.write("</ds_deathEvent>");
      }

      jOut.write("</disturbance>");
    }
  }

  /**
   * Writes the harvest data for the parameter file.  Does nothing if there
   * is no harvest data.
   * @param jOut FileWriter File to write to.
   * @param oPop Tree population object.
   * @throws java.io.IOException if there is a problem writing the file
   * @throws ModelException if there are unrecognized harvest cut types (should
   * never happen).
   */
  private void WriteHarvest(BufferedWriter jOut, TreePopulation oPop) throws
      java.
      io.IOException, ModelException {
    HarvestData oData;
    Cell oCell;
    int i, iNumObjects = mp_oHarvestCuts.size(), j, iTemp;

    //************************
     // Harvest
     //************************

      if (iNumObjects > 0 && GetBehaviorByKey("Harvest").m_bIsEnabled) {

        jOut.write("<harvest>");

        //Write each harvest object's data
        for (i = 0; i < iNumObjects; i++) {
          oData = (HarvestData) mp_oHarvestCuts.elementAt(i);

          jOut.write("<ha_cutEvent>");

          //Write each species
          iTemp = oData.GetNumberOfSpecies();
          for (j = 0; j < iTemp; j++) {
            jOut.write("<ha_applyToSpecies species=\"" +
                       oPop.GetSpeciesNameFromCode(oData.GetSpecies(j)) +
                       "\"/>");
          }

          //Write the timestep
          jOut.write("<ha_timestep>" + String.valueOf(oData.GetTimestep()) +
                     "</ha_timestep>");

          //Write the cut type
          jOut.write("<ha_cutType>");
          iTemp = oData.GetCutType();
          if (iTemp == GAP_CUT) {
            jOut.write("gap");
          }
          else if (iTemp == PARTIAL_CUT) {
            jOut.write("partial");
          }
          else if (iTemp == CLEAR_CUT) {
            jOut.write("clear");
          }
          else {
            ModelException oErr = new ModelException();
            oErr.iErrorCode = ErrorGUI.BAD_DATA;
            oErr.sCallingFunction = "JAVA";
            oErr.sMessage =
                "When writing parameter file, came across unrecognized " +
                "harvest cut type \"" + String.valueOf(iTemp) + "\".";
            throw oErr;
          }
          jOut.write("</ha_cutType>");

          //Write the cut amount type
          jOut.write("<ha_cutAmountType>");
          iTemp = oData.GetCutAmountType();
          if (iTemp == ABSOLUTE_BASAL_AREA) {
            jOut.write("absolute basal area");
          }
          else if (iTemp == ABSOLUTE_DENSITY) {
            jOut.write("absolute density");
          }
          else if (iTemp == PERCENTAGE_BASAL_AREA) {
            jOut.write("percent of basal area");
          }
          else if (iTemp == PERCENTAGE_DENSITY) {
            jOut.write("percent of density");
          }
          else {
            ModelException oErr = new ModelException();
            oErr.iErrorCode = ErrorGUI.BAD_DATA;
            oErr.sCallingFunction = "JAVA";
            oErr.sMessage =
                "When writing parameter file, came across unrecognized " +
                "harvest cut type \"" + String.valueOf(iTemp) + "\".";
            throw oErr;
          }
          jOut.write("</ha_cutAmountType>");

          //Write the cut ranges
          iTemp = oData.GetNumberOfCutRanges();
          for (j = 0; j < iTemp; j++) {

            jOut.write("<ha_dbhRange>");
            jOut.write("<ha_low>" + String.valueOf(oData.GetLowerBound(j)) +
                       "</ha_low>");
            jOut.write("<ha_high>" + String.valueOf(oData.GetUpperBound(j)) +
                       "</ha_high>");
            jOut.write("<ha_amountToCut>" +
                       String.valueOf(oData.GetCutAmount(j)) +
                       "</ha_amountToCut>");
            jOut.write("</ha_dbhRange>");
          }

          //Write the list of cells
          iTemp = oData.GetNumberOfCells();
          for (j = 0; j < iTemp; j++) {

            oCell = (Cell) oData.GetCell(j);

            jOut.write("<ha_applyToCell x=\"" + String.valueOf(oCell.GetX())
                       + "\" y=\"" + String.valueOf(oCell.GetY()) + "\"/>");
          }

          jOut.write("</ha_cutEvent>");
        }

        jOut.write("</harvest>");
      }

  }

  /**
   * Writes the harvest interface data for the parameter file.  Does nothing
   * if the harvest interface behavior is not being used.
   * @param jOut FileWriter File to write to.
   * @throws java.io.IOException if there is a problem writing the file
   */
  private void WriteHarvestInterface(BufferedWriter jOut) throws
      java.io.IOException {
    if (GetBehaviorByKey("Harvest Interface").m_bIsEnabled == false) return;

    jOut.write("<harvestInterface>");

    //Our parameters are not "assigned" to any behavior, so we have to use the
    //WorkerBase static method to write them.  Simply using WriteData would
    //prevent them from being written, since it belongs to BehaviorTypeBase and
    //checks to make sure data is applied to an enabled behavior.

    //User executable filename
    WorkerBase.WriteDataToFile(jOut, m_sHarvIntExecutable);

   //Filename of harvestable trees for SORTIE to write
   WorkerBase.WriteDataToFile(jOut, m_sHarvIntSORTIEOutFile);

   //Filename of trees to harvest for the user executable
   WorkerBase.WriteDataToFile(jOut, m_sHarvIntExecHarvestOutFile);

   //How often harvests occur
   WorkerBase.WriteDataToFile(jOut, m_iHarvIntHarvestPeriod);

   //Filename of trees to update for the user executable to write, if being
   //used
   if (m_sHarvIntExecUpdateOutFile.GetValue().length() > 0) {
     WorkerBase.WriteDataToFile(jOut, m_sHarvIntExecUpdateOutFile);
   }

   //Filename of batch parameters, if being used
   if (m_sHarvIntBatchParamsFile.GetValue().length() > 0) {
     WorkerBase.WriteDataToFile(jOut, m_sHarvIntBatchParamsFile);
     WorkerBase.WriteDataToFile(jOut, m_sHarvIntBatchSingleRunParamsFile);
   }

   //Arguments to pass to user executable, if being used
   if (m_sHarvIntExecArgs.GetValue().length() > 0) {
     WorkerBase.WriteDataToFile(jOut, m_sHarvIntExecArgs);
   }

   //List of tree data members to include in the files beyond the required
   //list, if being used
   if (mp_sHarvIntFileColumns.GetValue().size() > 0) {
     WorkerBase.WriteVectorValueToFile(jOut, mp_sHarvIntFileColumns);
   }

   jOut.write("</harvestInterface>");
  }

  /**
   * Validates the data prior to writing it.  What it does:
   * <ul>
   * <li>Causes all HarvestData objects to validate themselves.</li>
   * <li>If the Storms behavior is enabled, this makes sure that all storm
   * return intervals are greater than 0.</li>
   * <li>If the Storms behavior is enabled and the susceptibility is set to
   * "mapped", this makes sure that there are values for the grid object
   * "Storm Susceptibility" and they are greater than 0.</li>
   * <li>If the Storms behavior is enabled and the damage pattern is set to
   * "mapped", this makes sure that the grid resolution for the "Storm
   * Susceptibility" and "Storm Damage" grids is the same.</li>
   * <li>If the Storm Damage Applier behavior is enabled, this checks to make
   * sure the Storms behavior is enabled.</li>
   * <li>If the Storm Damage Applier behavior is enabled, this checks to make
   * sure the value in m_iNumYearsToHeal is positive.</li>
   * <li>If the Storm Killer behavior is enabled, this checks to make
   * sure that both the Storm and Storm Damage Applier behaviors are
   * enabled.</li>
   * <li>If the Storm Killer behavior is enabled, this checks to make
   * sure the value in m_iNumSnagYears is not negative.</li>
   * <li>If the Storm Damage Killer behavior is enabled, this makes sure that
   * the proportion of dead trees that tip up are all proportions.</li>
   * <li>If Selection Harvest is enabled, this makes sure that target basal
   * area is not less than 0 and cut ranges don't overlap</li>
   * <li>If Windstorm is enabled, this makes sure that:
   * <ul>
   * <li>Minimum DBH not negative for any value</li>
   * <li>Timestep to start storms not negative</li>
   * <li>Storm severities are all proportions</li>
   * <li>Some mortality behavior is applied to all trees</li>
   * </ul>
   * </ul>
   * @param oPop TreePopulation object
   * @throws ModelException if one of the data objects is not valid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>April 4, 2005:  Added new storm damage applier parameters (LEM)
   * <br>May 2, 2005:  Added new storm damage applier parameters (LEM)
   * <br>May 12, 2005:  Added selection harvest stuff (LEM)
   * <br>January 9, 2005:  Added storm damage stuff (LEM)
   * <br>March 19, 2006:  Added windstorm validation (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    HarvestData oData;
    Plot oPlot = m_oManager.GetPlot();
    int i, iNumObjects;

    //Disable the Harvest behavior if there are no harvest events
    //No longer - Selection Harvest may enable it
    //if (iNumObjects == 0) {
    //  GetBehaviorByKey("Harvest").m_bIsEnabled = false;
    // }

    //Validate each harvest event
    iNumObjects = mp_oHarvestCuts.size();
    for (i = 0; i < iNumObjects; i++) {
      oData = (HarvestData) mp_oHarvestCuts.elementAt(i);
      oData.ValidateCut(oPop, oPlot);
    }

    //Disable the Episodic Mortality behavior if there are no mortality episodes
    //events
    //if (iNumObjects == 0) {
    //  GetBehaviorByKey("Episodic Mortality").m_bIsEnabled = false;
    //}

    //Validate each Episodic Mortality event
    iNumObjects = mp_oMortEpisodes.size();
    for (i = 0; i < iNumObjects; i++) {
      oData = (HarvestData) mp_oMortEpisodes.elementAt(i);
      oData.ValidateCut(oPop, oPlot);
    }

    //Storms behavior validation
    if (GetBehaviorByKey("Storms").m_bIsEnabled) {

      //Make sure all the return intervals are non-negative
      MakeSureNonNegative(m_fClass1RtrnInt);
      MakeSureNonNegative(m_fClass2RtrnInt);
      MakeSureNonNegative(m_fClass3RtrnInt);
      MakeSureNonNegative(m_fClass4RtrnInt);
      MakeSureNonNegative(m_fClass5RtrnInt);
      MakeSureNonNegative(m_fClass6RtrnInt);
      MakeSureNonNegative(m_fClass7RtrnInt);
      MakeSureNonNegative(m_fClass8RtrnInt);
      MakeSureNonNegative(m_fClass9RtrnInt);
      MakeSureNonNegative(m_fClass10RtrnInt);

      //If the damage pattern is mapped, make sure there are values for the
      //"Storm Susceptibility" grid and that they are all > 0
      if (m_iSusceptibility.GetStringValue().equalsIgnoreCase("mapped")) {
        Grid oSusceptibility = m_oManager.GetGridByName("Storm Susceptibility");
        if (oSusceptibility == null) {
          //"Storm Susceptibility" is missing - shouldn't happen, but a
          //good double-check
          ModelException oErr = new ModelException(ErrorGUI.DATA_MISSING,
              "JAVA",
              "The grid \"Storm Susceptibility\" is required for mapped storm damage patterns.");
          throw (oErr);
        }

        iNumObjects = oSusceptibility.mp_oGridVals.size();
        if (iNumObjects == 0) {
          //Map is missing for "Storm Susceptibility"
          ModelException oErr = new ModelException(ErrorGUI.DATA_MISSING,
              "JAVA",
              "A map of values for the grid \"Storm Susceptibility\" is required for mapped storm damage patterns.");
          throw (oErr);
        }

        for (i = 0; i < iNumObjects; i++) {
          GridValue oVal = (GridValue) oSusceptibility.mp_oGridVals.elementAt(i);
          if (oVal.mp_fFloats == null || oVal.mp_fFloats[0] == null ||
              oVal.mp_fFloats[0].floatValue() < 0) {
            ModelException oErr = new ModelException(ErrorGUI.DATA_MISSING,
                "JAVA",
                "All values in the map for the grid \"Storm Susceptibility\" must be greater than 0.");
            throw (oErr);
          }
        }

        //Get the grid "Storm Damage" and make sure it has the same resolution
        //as "Storm Susceptibility"
        Grid oDamage = m_oManager.GetGridByName("Storm Damage");
        if (oDamage != null) {
          if (oDamage.m_bEdited) {
            if (java.lang.Math.abs(oDamage.GetXCellLength() -
                                   oSusceptibility.GetXCellLength()) > 0.01 ||
                java.lang.Math.abs(oDamage.GetYCellLength() -
                                   oSusceptibility.GetYCellLength()) > 0.01) {
              ModelException oErr = new ModelException(ErrorGUI.BAD_DATA,
                  "JAVA",
                  "The grids \"Storm Susceptibility\" and \"Storm Damage\" " +
                  "must have matching grid cell resolutions.");
              throw (oErr);

            }
          }
        }
      }
    }

    if (GetBehaviorByKey("Storm Damage Applier").m_bIsEnabled) {
      if (GetBehaviorByKey("Storms").m_bIsEnabled == false) {
        ModelException oErr = new ModelException(ErrorGUI.BAD_DATA,
                                                 "JAVA",
                                                 "When the behavior \"Storm Damage Applier\" is enabled, " +
                                                 "the behavior \"Storms\" must be enabled as well.");
        throw (oErr);
      }
      MakeSurePositive(m_iNumYearsToHeal);
    }

    if (GetBehaviorByKey("Storm Killer").m_bIsEnabled) { //LEM 01/09/06
      Behavior oKiller = GetBehaviorByKey("Storm Killer"),
          oDamageApplier = GetBehaviorByKey("Storm Damage Applier");
      if (oDamageApplier.m_bIsEnabled == false) {
        ModelException oErr = new ModelException(ErrorGUI.BAD_DATA,
                                                 "JAVA",
                                                 "When the behavior \"Storm Killer\" is enabled, " +
                                                 "the behavior \"Storm Damage Applier\" must be enabled as well.");
        throw (oErr);
      }

      //Make sure every species/type combo for storm killer has also been
      //assigned to storm damage applier
      SpeciesTypeCombo oKillerCombo, oDamageCombo;
      int j;
      boolean bFound;
      for (i = 0; i < oKiller.GetNumberOfCombos(); i++) {
        bFound = false;
        oKillerCombo = oKiller.GetSpeciesTypeCombo(i);
        for (j = 0; j < oDamageApplier.GetNumberOfCombos(); j++) {
          oDamageCombo = oDamageApplier.GetSpeciesTypeCombo(j);
          if (oKillerCombo.equals(oDamageCombo)) {
            bFound = true;
            break;
          }
        }
        if (bFound == false) {
          ModelException oErr = new ModelException(ErrorGUI.BAD_DATA,
                                                 "JAVA",
                                                 "All trees to which the behavior \"Storm Killer\" is applied " +
                                                 "must also have the behavior \"Storm Damage Applier\" applied as well.");
        throw (oErr);

        }
      }

      boolean[] p_bAppliesTo = GetBehaviorByKey("Storm Killer").
          GetWhichSpeciesUsed(oPop);
      MakeSureAllAreProportions(mp_fStmPropFullTipUp, p_bAppliesTo);
      if (m_oManager.GetSnagAwareness() == true) {
        MakeSurePositive(m_iNumSnagYears);
      }
    }

    //LEM 05-12-05
    if (GetBehaviorByKey("Selection Harvest").m_bIsEnabled) {

      GetBehaviorByKey("Harvest").m_bIsEnabled = true;

      MakeSureNonNegative(m_fCut1BA);
      MakeSureNonNegative(m_fCut2BA);
      MakeSureNonNegative(m_fCut3BA);
      MakeSureNonNegative(m_fCut4BA);

      //Make sure the upper bound of each cut range is larger than the lower
      //bound
      if (m_fCut1HiDBH.GetValue() < m_fCut1LoDBH.GetValue() ||
          m_fCut2HiDBH.GetValue() < m_fCut2LoDBH.GetValue() ||
          m_fCut3HiDBH.GetValue() < m_fCut3LoDBH.GetValue() ||
          m_fCut4HiDBH.GetValue() < m_fCut4LoDBH.GetValue()) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA", "Selection Harvest cut range upper DBHs must be greater than corresponding lower DBHs.");
      }
    }

    //LEM 03/19/06
    if (GetBehaviorByKey("Windstorm").m_bIsEnabled) {
      Behavior oWindstorm = GetBehaviorByKey("Windstorm");
      //Make sure all storm severity values are proportions
      MakeSureIsProportion(m_fWindstormReturnInt1Severity);
      MakeSureIsProportion(m_fWindstormReturnInt5Severity);
      MakeSureIsProportion(m_fWindstormReturnInt10Severity);
      MakeSureIsProportion(m_fWindstormReturnInt20Severity);
      MakeSureIsProportion(m_fWindstormReturnInt40Severity);
      MakeSureIsProportion(m_fWindstormReturnInt80Severity);
      MakeSureIsProportion(m_fWindstormReturnInt160Severity);
      MakeSureIsProportion(m_fWindstormReturnInt320Severity);
      MakeSureIsProportion(m_fWindstormReturnInt640Severity);
      MakeSureIsProportion(m_fWindstormReturnInt1280Severity);
      MakeSureIsProportion(m_fWindstormReturnInt2560Severity);

      //Make sure the timestep to start storms is not negative
      MakeSureNonNegative(m_iWindstormTimestepToStartStorms);

      //Make sure the SST periodicity is not 0
      MakeSureNonZero(m_fWindstormSSTPeriod);

      //Make sure minimum DBH values are not negative
      boolean[] p_bApplies = oWindstorm.GetWhichSpeciesUsed(oPop);
      MakeSureAllNonNegative(mp_fWindstormMinDBH, p_bApplies);

      //Make sure there's a mortality behavior applied to every tree to
      //which windstorm is applied
      Behavior[] p_oMortBeh = m_oManager.GetMortalityBehaviors().GetAllBehaviors();
      SpeciesTypeCombo oWindstormCombo, oMortCombo;
      int iBeh, j, k, iNumWindstormCombos = oWindstorm.GetNumberOfCombos(),
          iNumMortCombos;
      boolean bFound;
      for (i = 0; i < iNumWindstormCombos; i++) {
        bFound = false;
        oWindstormCombo = oWindstorm.GetSpeciesTypeCombo(i);
        for (iBeh = 0; iBeh < p_oMortBeh.length; iBeh++) {
          iNumMortCombos = p_oMortBeh[iBeh].GetNumberOfCombos();
          for (j = 0; j < iNumMortCombos; j++) {
            oMortCombo = p_oMortBeh[iBeh].GetSpeciesTypeCombo(j);
            if (oWindstormCombo.equals(oMortCombo)) {
              bFound = true;
              break;
            }
          }
          if (bFound) break;
        }
        if (bFound == false) {
          ModelException oErr = new ModelException(ErrorGUI.BAD_DATA,
                                                 "JAVA",
                                                 "All trees to which the behavior \"" +
                                                 oWindstorm.GetDescriptor() + "\" is applied " +
                                                 "must also have a mortality behavior applied as well.");
        throw (oErr);

        }
      }

    }
  }

  /**
   * Overridden because disturbance writing is special.
   * @param jOut File to write to.
   * @param oPop TreePopulation object
   * @throws javawrapper.ModelException if there's a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>May 13, 2004:  Added episodic mortality writing.  (LEM)
   * <br>May 12, 2005:  Added selection harvest writing (LEM)
   * <br>March 19, 2006:  Added windstorm parameter writing (LEM)
   */
  public void WriteXML(BufferedWriter jOut, TreePopulation oPop) throws
      ModelException {
    try {

      WriteHarvest(jOut, oPop);
      WriteMortalityEpisode(jOut, oPop);
      WriteStorm(jOut);
      WriteSelectionHarvest(jOut); //LEM 05/12/05
      WriteWindstorm(jOut); //LEM 03/16/06
      WriteHarvestInterface(jOut); //LEM 09-29-06

    }
    catch (java.io.IOException e) {
      ModelException oErr = new ModelException();
      oErr.iErrorCode = ErrorGUI.BAD_FILE;
      oErr.sCallingFunction = "JAVA";
      oErr.sMessage = "There was a problem writing the parameter file.";
      throw (oErr);

    }
  }

  /**
   * Sets up the grid objects.
   * @param oPop TreePopulation object.
   * @throws javawrapper.ModelException passthrough only.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>May 13, 2004:  Added episodic mortality grid.  (LEM)
   * <br>April 4, 2005:  Defaulted new storm params (LEM)
   */
  public void DoSetup(TreePopulation oPop) throws javawrapper.ModelException {

    int iNumSpecies = oPop.GetNumberOfSpecies(),
        i, j, k, m;

    //mp_oGrids = new Grid[5];
    Grid oNewGrid;

    //********************************
     //Harvest grid
     //********************************

    String sGridName = "Harvest Results";

    //The accessible data members are one for each cut range and species
    String[] p_sBaseMembers = {
        "Cut Density", "Cut Basal Area"};
    DataMember[] p_oDataMembers = new DataMember[p_sBaseMembers.length *
        iNumSpecies *
        HarvestData.NUMBER_ALLOWED_CUT_RANGES + 1];

    p_oDataMembers[0] = new DataMember("Harvest Type", "Harvest Type",
                                       DataMember.INTEGER);

    m = 1;
    for (i = 0; i < p_sBaseMembers.length; i++) {
      if (p_sBaseMembers[i].equals("Cut Density")) {
        for (j = 0; j < iNumSpecies; j++) {
          for (k = 0; k < HarvestData.NUMBER_ALLOWED_CUT_RANGES; k++) {
            p_oDataMembers[m] = new DataMember(p_sBaseMembers[i] + ", " +
                                               oPop.GetSpeciesNameFromCode(j).
                                               replace('_', ' ')
                                               + ", cut range " + k,
                                               p_sBaseMembers[i] + "_" + k +
                                               "_" +
                                               j,
                                               DataMember.INTEGER);
            m++;
          }
        }
      }
      else {
        for (j = 0; j < iNumSpecies; j++) {
          for (k = 0; k < HarvestData.NUMBER_ALLOWED_CUT_RANGES; k++) {
            p_oDataMembers[m] = new DataMember(p_sBaseMembers[i] + ", " +
                                               oPop.GetSpeciesNameFromCode(j).
                                               replace('_', ' ')
                                               + ", cut range " + k,
                                               p_sBaseMembers[i] + "_" + k +
                                               "_" +
                                               j,
                                               DataMember.FLOAT);
            m++;
          }
        }
      }
    }

    //Create our grid
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Add it to the harvest behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Harvest").AddGrid(oNewGrid);

    //********************************
     //Episodic Mortality grid
     //********************************

    sGridName = "Mortality Episode Results";

    //The accessible data members are one for each cut range and species
    p_oDataMembers = new DataMember[p_sBaseMembers.length *
        iNumSpecies *
        HarvestData.NUMBER_ALLOWED_CUT_RANGES];

    m = 0;
    for (i = 0; i < p_sBaseMembers.length; i++) {
      if (p_sBaseMembers[i].equals("Cut Density")) {
        for (j = 0; j < iNumSpecies; j++) {
          for (k = 0; k < HarvestData.NUMBER_ALLOWED_CUT_RANGES; k++) {
            p_oDataMembers[m] = new DataMember(p_sBaseMembers[i] + ", " +
                                               oPop.GetSpeciesNameFromCode(j).
                                               replace('_', ' ') +
                                               ", cut range " + k,
                                               p_sBaseMembers[i] + "_" + k +
                                               "_" + j,
                                               DataMember.INTEGER);
            m++;
          }
        }
      }
      else {
        for (j = 0; j < iNumSpecies; j++) {
          for (k = 0; k < HarvestData.NUMBER_ALLOWED_CUT_RANGES; k++) {
            p_oDataMembers[m] = new DataMember(p_sBaseMembers[i] + ", " +
                                               oPop.GetSpeciesNameFromCode(j).
                                               replace('_', ' ')
                                               + ", cut range " + k,
                                               p_sBaseMembers[i] + "_" + k +
                                               "_" + j,
                                               DataMember.FLOAT);
            m++;
          }
        }
      }
    }

    //Create our grid
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Add it to the episodic mortality behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Episodic Mortality").AddGrid(oNewGrid);

    //********************************
     //Storm Damage grid
     //********************************

    sGridName = "Storm Damage";

    p_oDataMembers = new DataMember[2];
    p_oDataMembers[0] = new DataMember("Damage Index", "dmg_index",
                                       DataMember.FLOAT);
    p_oDataMembers[1] = new DataMember("Time Since Last Storm", "stormtime",
                                       DataMember.FLOAT);

    //Create our grid
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Add to the Storms behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Storms").AddGrid(oNewGrid);

    //********************************
     //Storm Susceptibility grid
     //********************************

    sGridName = "Storm Susceptibility";

    //The accessible data members are one for each cut range and species
    p_oDataMembers = new DataMember[1];
    p_oDataMembers[0] = new DataMember("Susceptibility", "index",
                                       DataMember.FLOAT);

    //Create our grid
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Add to the Storms behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Storms").AddGrid(oNewGrid);

    //********************************
     //Windstorm Results grid
     //********************************
    p_oDataMembers = new DataMember[(2 * iNumSpecies) + 1];
    Plot p_oPlot = m_oManager.GetPlot();

    sGridName = "Windstorm Results";

    //The accessible data members are storm severity, and a basal area and
    //density killed for each species
    for (i = 0; i < iNumSpecies; i++) {

      p_oDataMembers[i] = new DataMember("Basal Area Dead For " +
                                         oPop.GetSpeciesNameFromCode(i).
                                         replace('_', ' '),
                                         "ba_" + i,
                                         DataMember.FLOAT);
    }

    for (i = 0; i < iNumSpecies; i++) {

      p_oDataMembers[i+iNumSpecies] = new DataMember("Density Dead For " +
                                         oPop.GetSpeciesNameFromCode(i).
                                         replace('_', ' '),
                                         "density_" + i,
                                         DataMember.FLOAT);
    }

    p_oDataMembers[2 * iNumSpecies] = new DataMember("Storm Severity", "severity", DataMember.FLOAT);


    //Create our grid
    oNewGrid = new Grid(sGridName, null, p_oDataMembers,
                            p_oPlot.GetPlotXLength(), p_oPlot.GetPlotYLength());

    //Assign the grid to the windstorm behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Windstorm").AddGrid(oNewGrid);

  }

  /**
   * Accepts an XML parent tag (empty, no data) from the parser.  This method
   * watches for the following tags:
   * <ul>
   * <li>ha_applyToCell</li>
   * <li>ha_applyToSpecies</li>
   * <li>ha_dbhRange</li>
   * <li>ha_cutEvent</li>
   * <li>ds_applyToCell</li>
   * <li>ds_applyToSpecies</li>
   * <li>ds_dbhRange</li>
   * <li>ds_deathEvent</li>
   * <li>SelectionHarvest</li>
   * <li>sha_CutRange</li>
   * </ul>
   * @param sXMLTag The XML tag.
   * @param oAttributes The attributes of this object.
   * @throws ModelException if there is a problem reading this data.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>May 12, 2004: Created (LEM)
   */
  public void ReadXMLParentTag(String sXMLTag, Attributes oAttributes) throws
      ModelException {

    if (sXMLTag.equals("ds_applyToCell")) {

      //Get the X and Y values
      int iX = new Integer(oAttributes.getValue("x")).intValue(),
          iY = new Integer(oAttributes.getValue("y")).intValue();

      //Assign them to the newest mortality episode
      HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes.elementAt(
          mp_oMortEpisodes.size() - 1);
      oMortEpisode.AddCell(iX, iY, m_oManager.GetPlot());

    }
    else if (sXMLTag.equals("ds_applyToSpecies")) {

      //Extract species
      TreePopulation oPop = m_oManager.GetTreePopulation();
      int iSpecies = oPop.GetSpeciesCodeFromName(oAttributes.getValue("species"));

      //Get the last harvest event
      if (mp_oMortEpisodes.size() == 0) {
        ModelException oErr = new ModelException();
        oErr.iErrorCode = ErrorGUI.BAD_DATA;
        oErr.sCallingFunction = "JAVA";
        oErr.sMessage = "Mortality episode parameters are incorrectly specified in the parameter file.  Found tag \"ds_applyToSpecies\" outside of disturbance event tag.";
        throw (oErr);
      }
      HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes.elementAt(
          mp_oMortEpisodes.size() - 1);

      //Add this species to that event
      oMortEpisode.AddSpecies(iSpecies);

    }
    else if (sXMLTag.equals("ds_dbhRange")) {

      //Reset our placeholders for cut range
      m_iCutRangeAmount = -1;
      m_iCutRangeLowerBound = -1;
      m_iCutRangeUpperBound = -1;
    }
    else if (sXMLTag.equals("ha_applyToCell")) {

      //Get the X and Y values
      int iX = new Integer(oAttributes.getValue("x")).intValue(),
          iY = new Integer(oAttributes.getValue("y")).intValue();

      //Assign them to the newest harvest event
      HarvestData oHarvest = (HarvestData) mp_oHarvestCuts.elementAt(
          mp_oHarvestCuts.size() - 1);
      oHarvest.AddCell(iX, iY, m_oManager.GetPlot());

    }
    else if (sXMLTag.equals("ha_applyToSpecies")) {

      //Extract species
      TreePopulation oPop = m_oManager.GetTreePopulation();
      int iSpecies = oPop.GetSpeciesCodeFromName(oAttributes.getValue("species"));

      //Get the last harvest event
      HarvestData oHarvest = (HarvestData) mp_oHarvestCuts.elementAt(
          mp_oHarvestCuts.size() - 1);

      //Add this species to that event
      oHarvest.AddSpecies(iSpecies);

    }
    else if (sXMLTag.equals("ha_dbhRange")) {

      //Reset our placeholders for cut range
      m_iCutRangeAmount = -1;
      m_iCutRangeLowerBound = -1;
      m_iCutRangeUpperBound = -1;
    }
    else if (sXMLTag.equals("ds_deathEvent")) {

      //Create a new mortality episode
      HarvestData oMortEpisode = new HarvestData();
      mp_oMortEpisodes.add(oMortEpisode);

      //Set it to partial cut - what all mortality episodes get
      oMortEpisode.SetCutType(PARTIAL_CUT);
    }
    else if (sXMLTag.equals("ha_cutEvent")) {

      //Create a new harvest event
      HarvestData oHarvest = new HarvestData();
      mp_oHarvestCuts.add(oHarvest);
    }
    else if (sXMLTag.equals("sha_CutRange")) { //LEM 05-12-05
      m_iSelectionHarvestCutRange++;
    }
    else if (sXMLTag.equals("harvest")) {

      //Enable the harvest behavior
      GetBehaviorByKey("Harvest").m_bIsEnabled = true;
    }
    else if (sXMLTag.equals("disturbance")) {

      //Enable the episodic mortality behavior
      GetBehaviorByKey("Episodic Mortality").m_bIsEnabled = true;
    }
    else if (sXMLTag.equals("SelectionHarvest")) { //LEM 05-12-05

      //Enable the harvest behavior and selection harvest behavior
      GetBehaviorByKey("Harvest").m_bIsEnabled = true;
      GetBehaviorByKey("Selection Harvest").m_bIsEnabled = true;
      m_iSelectionHarvestCutRange = 0;
    }
    super.ReadXMLParentTag(sXMLTag, oAttributes);
  }

  /**
   * This method looks for the following tags:
   * <ul>
   * <li>ds_timestep
   * <li>ds_cutAmountType
   * <li>ds_low
   * <li>ds_high
   * <li>ds_amountToCut
   * <li>ha_timestep
   * <li>ha_cutType
   * <li>ha_cutAmountType
   * <li>ha_low
   * <li>ha_high
   * <li>ha_amountToCut
   * </ul>
   * @param sXMLTag XML tag of data object whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param oAttributes Attributes of the object.  Ignored, but may be needed
   * by overriding objects.
   * @param oData Data value appropriate to the data type
   * @return true if the value was set successfully; false if the value could
   * not be found.  (This would not be an error, because I need a way to
   * cycle through the objects until one of the objects comes up with a
   * match.)
   * @throws ModelException if the value could not be assigned to the data
   * object, or if the cut type or cut type amount values are unrecognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean SetSingleValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        Attributes oAttributes,
                                        Object oData) throws
      ModelException {
    if (sXMLTag != null) {
      if (sXMLTag.equals("ds_timestep")) {

        //Get the timestep value
        int iTimestep = new Integer(String.valueOf(oData)).intValue();

        //Get the last harvest event and set this as the timestep
        HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes.elementAt(
            mp_oMortEpisodes.size() - 1);
        oMortEpisode.SetTimestep(iTimestep);

        return true;
      }
      else if (sXMLTag.equals("ds_cutAmountType")) {

        //Get the cut amount type string
        String sCut = String.valueOf(oData);

        //Get the last harvest event
        HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes.elementAt(
            mp_oMortEpisodes.size() - 1);

        if (sCut.equalsIgnoreCase("percent of basal area")) {
          oMortEpisode.SetCutAmountType(PERCENTAGE_BASAL_AREA);
        }
        else if (sCut.equalsIgnoreCase("percent of density")) {
          oMortEpisode.SetCutAmountType(PERCENTAGE_DENSITY);
        }
        else if (sCut.equalsIgnoreCase("absolute basal area")) {
          oMortEpisode.SetCutAmountType(ABSOLUTE_BASAL_AREA);
        }
        else if (sCut.equalsIgnoreCase("absolute density")) {
          oMortEpisode.SetCutAmountType(ABSOLUTE_DENSITY);
        }
        else {
          //Unrecognized - error
          ModelException oErr = new ModelException();
          oErr.iErrorCode = ErrorGUI.BAD_DATA;
          oErr.sCallingFunction = "JAVA";
          oErr.sMessage =
              "Episodic mortality event in file has an unrecognized cut amount type - " +
              sCut + ".";
          throw (oErr);
        }

        return true;
      }
      else if (sXMLTag.equals("ds_low")) {

        //Extract the value
        float fVal = new Float(String.valueOf(oData)).floatValue();

        //Check to see whether the high and amount values have already been
        //read
        if (m_iCutRangeAmount > -1 && m_iCutRangeUpperBound > -1) {

          //Assign them
          HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes.elementAt(
              mp_oMortEpisodes.size() - 1);

          oMortEpisode.AddCutRange(fVal, m_iCutRangeUpperBound,
                                   m_iCutRangeAmount);
          m_iCutRangeAmount = -1;
          m_iCutRangeUpperBound = -1;
          m_iCutRangeLowerBound = -1;
        }

        //Not time to assign - stash this value
        m_iCutRangeLowerBound = fVal;

        return true;
      }
      else if (sXMLTag.equals("ds_high")) {
        //Extract the value
        float fVal = new Float(String.valueOf(oData)).floatValue();

        //Check to see whether the low and amount values have already been
        //read
        if (m_iCutRangeAmount > -1 && m_iCutRangeLowerBound > -1) {

          //Assign them
          HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes.elementAt(
              mp_oMortEpisodes.size() - 1);

          oMortEpisode.AddCutRange(m_iCutRangeLowerBound, fVal,
                                   m_iCutRangeAmount);
          m_iCutRangeAmount = -1;
          m_iCutRangeUpperBound = -1;
          m_iCutRangeLowerBound = -1;
        }

        //Not time to assign - stash this value
        m_iCutRangeUpperBound = fVal;

        return true;
      }
      else if (sXMLTag.equals("ds_amountToCut")) {
        //Extract the value
        float fVal = new Float(String.valueOf(oData)).floatValue();

        //Check to see whether the low and high bounds have already been
        //read
        if (m_iCutRangeUpperBound > -1 && m_iCutRangeLowerBound > -1) {

          //Assign them
          HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes.elementAt(
              mp_oMortEpisodes.size() - 1);

          oMortEpisode.AddCutRange(m_iCutRangeLowerBound, m_iCutRangeUpperBound,
                                   fVal);
          m_iCutRangeAmount = -1;
          m_iCutRangeUpperBound = -1;
          m_iCutRangeLowerBound = -1;
        }

        //Not time to assign - stash this value
        m_iCutRangeAmount = fVal;

        return true;
      }
      else if (sXMLTag.equals("ha_timestep")) {

        //Get the timestep value
        int iTimestep = new Integer(String.valueOf(oData)).intValue();

        //Get the last harvest event and set this as the timestep
        HarvestData oHarvest = (HarvestData) mp_oHarvestCuts.elementAt(
            mp_oHarvestCuts.size() - 1);
        oHarvest.SetTimestep(iTimestep);

        return true;
      }
      else if (sXMLTag.equals("ha_cutType")) {

        //Get the cut type string
        String sCut = String.valueOf(oData);

        //Get the last harvest event
        HarvestData oHarvest = (HarvestData) mp_oHarvestCuts.elementAt(
            mp_oHarvestCuts.size() - 1);

        if (sCut.equalsIgnoreCase("gap")) {
          oHarvest.SetCutType(GAP_CUT);
        }
        else if (sCut.equalsIgnoreCase("partial")) {
          oHarvest.SetCutType(PARTIAL_CUT);
        }
        else if (sCut.equalsIgnoreCase("clear")) {
          oHarvest.SetCutType(CLEAR_CUT);
        }
        else {
          //Unrecognized - error
          ModelException oErr = new ModelException();
          oErr.iErrorCode = ErrorGUI.BAD_DATA;
          oErr.sCallingFunction = "JAVA";
          oErr.sMessage = "Harvest cut in file has an unrecognized cut type - " +
              sCut + ".";
          throw (oErr);
        }

        return true;
      }
      else if (sXMLTag.equals("ha_cutAmountType")) {

        //Get the cut amount type string
        String sCut = String.valueOf(oData);

        //Get the last harvest event
        HarvestData oHarvest = (HarvestData) mp_oHarvestCuts.elementAt(
            mp_oHarvestCuts.size() - 1);

        if (sCut.equalsIgnoreCase("percent of basal area")) {
          oHarvest.SetCutAmountType(PERCENTAGE_BASAL_AREA);
        }
        else if (sCut.equalsIgnoreCase("percent of density")) {
          oHarvest.SetCutAmountType(PERCENTAGE_DENSITY);
        }
        else if (sCut.equalsIgnoreCase("absolute basal area")) {
          oHarvest.SetCutAmountType(ABSOLUTE_BASAL_AREA);
        }
        else if (sCut.equalsIgnoreCase("absolute density")) {
          oHarvest.SetCutAmountType(ABSOLUTE_DENSITY);
        }
        else {
          //Unrecognized - error
          ModelException oErr = new ModelException();
          oErr.iErrorCode = ErrorGUI.BAD_DATA;
          oErr.sCallingFunction = "JAVA";
          oErr.sMessage =
              "Harvest cut in file has an unrecognized cut amount type - " +
              sCut + ".";
          throw (oErr);
        }

        return true;
      }
      else if (sXMLTag.equals("ha_low")) {

        //Extract the value
        float fVal = new Float(String.valueOf(oData)).floatValue();

        //Check to see whether the high and amount values have already been
        //read
        if (m_iCutRangeAmount > -1 && m_iCutRangeUpperBound > -1) {

          //Assign them
          HarvestData oHarvest = (HarvestData) mp_oHarvestCuts.elementAt(
              mp_oHarvestCuts.size() - 1);

          oHarvest.AddCutRange(fVal, m_iCutRangeUpperBound, m_iCutRangeAmount);
          m_iCutRangeAmount = -1;
          m_iCutRangeUpperBound = -1;
          m_iCutRangeLowerBound = -1;
        }

        //Not time to assign - stash this value
        m_iCutRangeLowerBound = fVal;

        return true;
      }
      else if (sXMLTag.equals("ha_high")) {
        //Extract the value
        float fVal = new Float(String.valueOf(oData)).floatValue();

        //Check to see whether the low and amount values have already been
        //read
        if (m_iCutRangeAmount > -1 && m_iCutRangeLowerBound > -1) {

          //Assign them
          HarvestData oHarvest = (HarvestData) mp_oHarvestCuts.elementAt(
              mp_oHarvestCuts.size() - 1);

          oHarvest.AddCutRange(m_iCutRangeLowerBound, fVal, m_iCutRangeAmount);
          m_iCutRangeAmount = -1;
          m_iCutRangeUpperBound = -1;
          m_iCutRangeLowerBound = -1;
        }

        //Not time to assign - stash this value
        m_iCutRangeUpperBound = fVal;

        return true;
      }
      else if (sXMLTag.equals("ha_amountToCut")) {
        //Extract the value
        float fVal = new Float(String.valueOf(oData)).floatValue();

        //Check to see whether the low and high bounds have already been
        //read
        if (m_iCutRangeUpperBound > -1 && m_iCutRangeLowerBound > -1) {

          //Assign them
          HarvestData oHarvest = (HarvestData) mp_oHarvestCuts.elementAt(
              mp_oHarvestCuts.size() - 1);

          oHarvest.AddCutRange(m_iCutRangeLowerBound, m_iCutRangeUpperBound,
                               fVal);
          m_iCutRangeAmount = -1;
          m_iCutRangeUpperBound = -1;
          m_iCutRangeLowerBound = -1;
        }

        //Not time to assign - stash this value
        m_iCutRangeAmount = fVal;

        return true;
      }
      else if (sXMLTag.equals("sha_target_BA")) {
        //Extract the value
        float fVal = new Float(String.valueOf(oData)).floatValue();

        //Assuming we have a valid cut range counter, assign this to the
        //appropriate variable
        if (m_iSelectionHarvestCutRange == 1) {
          m_fCut1BA.SetValue(fVal);
        }
        if (m_iSelectionHarvestCutRange == 2) {
          m_fCut2BA.SetValue(fVal);
        }
        if (m_iSelectionHarvestCutRange == 3) {
          m_fCut3BA.SetValue(fVal);
        }
        if (m_iSelectionHarvestCutRange == 4) {
          m_fCut4BA.SetValue(fVal);

        }
        return true;
      }
      else if (sXMLTag.equals("sha_loDBH")) {
        //Extract the value
        float fVal = new Float(String.valueOf(oData)).floatValue();

        //Assuming we have a valid cut range counter, assign this to the
        //appropriate variable
        if (m_iSelectionHarvestCutRange == 1) {
          m_fCut1LoDBH.SetValue(fVal);
        }
        if (m_iSelectionHarvestCutRange == 2) {
          m_fCut2LoDBH.SetValue(fVal);
        }
        if (m_iSelectionHarvestCutRange == 3) {
          m_fCut3LoDBH.SetValue(fVal);
        }
        if (m_iSelectionHarvestCutRange == 4) {
          m_fCut4LoDBH.SetValue(fVal);

        }
        return true;
      }
      else if (sXMLTag.equals("sha_hiDBH")) {
        //Extract the value
        float fVal = new Float(String.valueOf(oData)).floatValue();

        //Assuming we have a valid cut range counter, assign this to the
        //appropriate variable
        if (m_iSelectionHarvestCutRange == 1) {
          m_fCut1HiDBH.SetValue(fVal);
        }
        if (m_iSelectionHarvestCutRange == 2) {
          m_fCut2HiDBH.SetValue(fVal);
        }
        if (m_iSelectionHarvestCutRange == 3) {
          m_fCut3HiDBH.SetValue(fVal);
        }
        if (m_iSelectionHarvestCutRange == 4) {
          m_fCut4HiDBH.SetValue(fVal);

        }
        return true;
      }
    }
    return super.SetSingleValueByXMLTag(sXMLTag, sXMLParentTag, oAttributes,
                                        oData);

  }

  /**
   * Checks episodic disturbance settings upon change of species.  This will
   * remove any deleted species.
   * @param iOldNumSpecies says how many species there used to be.
   * @param p_iIndexer is an array, sized to the new number of species.  For
   * each bucket (representing the index number of a species on the new list),
   * the value is either the index of that same species in the old species
   * list, or -1 if the species is new.
   * @throws ModelException if anything goes wrong.
   */
  public void ChangeOfSpecies(int iOldNumSpecies, int[] p_iIndexer) throws
      ModelException {
    HarvestData oEpisode;
    int[] p_iOldCutSpecies;
    int i, j, k;

    //Go through all the harvests and check for species that have been removed
    for (i = 0; i < mp_oHarvestCuts.size(); i++) {
      oEpisode = (HarvestData) mp_oHarvestCuts.elementAt(i);
      //Make a list of the species for this cut
      p_iOldCutSpecies = new int[oEpisode.GetNumberOfSpecies()];
      for (j = 0; j < p_iOldCutSpecies.length; j++) {
        p_iOldCutSpecies[j] = oEpisode.GetSpecies(j);
      }
      //Now remove all the old species
      oEpisode.ClearSpecies();

      //Now add back the species from the copy we saved
      for (j = 0; j < p_iOldCutSpecies.length; j++) {

        //Look for this species in the new species list - add the new species
        //number if we find it
        for (k = 0; k < p_iIndexer.length; k++) {
          if (p_iIndexer[k] == p_iOldCutSpecies[j]) {
            //Yep, this species is still on the list - change the species
            //number
            oEpisode.AddSpecies(k);
            break;
          }
        }
      }

      //If this harvest event has no species left, delete it
      if (oEpisode.GetNumberOfSpecies() == 0) {
        mp_oHarvestCuts.remove(i);
        i--;
      }
    }

    //Repeat with episodic mortality
    for (i = 0; i < mp_oMortEpisodes.size(); i++) {
      oEpisode = (HarvestData) mp_oMortEpisodes.elementAt(i);
      //Make a list of the species for this cut
      p_iOldCutSpecies = new int[oEpisode.GetNumberOfSpecies()];
      for (j = 0; j < p_iOldCutSpecies.length; j++) {
        p_iOldCutSpecies[j] = oEpisode.GetSpecies(j);
      }
      //Now remove all the old species
      oEpisode.ClearSpecies();

      //Now add back the species from the copy we saved
      for (j = 0; j < p_iOldCutSpecies.length; j++) {

        //Look for this species in the new species list - add the new species
        //number if we find it
        for (k = 0; k < p_iIndexer.length; k++) {
          if (p_iIndexer[k] == p_iOldCutSpecies[j]) {
            //Yep, this species is still on the list - change the species
            //number
            oEpisode.AddSpecies(k);
            break;
          }
        }
      }

      //If this episodic event has no species left, delete it
      if (oEpisode.GetNumberOfSpecies() == 0) {
        mp_oMortEpisodes.remove(i);
        i--;
      }
    }
  }

  /**
   * Overridden for backwards compatibility after changing XML tag names.  This
   * was done in version 6.06 and can be removed after one version, since it
   * was done to a behavior not widely in production at the time of release.
   * @param sXMLTag Parent XML tag of data vector whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param p_oData Vector of data values appropriate to the data type
   * @param p_sChildXMLTags The XML tags of the child elements
   * @param p_bAppliesTo Array of booleans saying which of the vector values
   * should be set.  This is important in the case of species-specifics - the
   * vector index is the species number but not all species are set.
   * @param oParentAttributes Attributes of parent tag.  May be useful when
   * overridding this for unusual tags.
   * @param p_oAttributes Attributes passed from parser.  This may be needed
   * when overriding this function.  Basic species-specific values are
   * already handled by this function.
   * @return true if the value was set successfully; false if the value could
   * not be found.  (This would not be an error, because I need a way to
   * cycle through the objects until one of the objects comes up with a
   * match.)  If a match to a data object is made via XML tag, but the found
   * object is not a ModelVector, this returns false.
   * @throws ModelException if the value could not be assigned to the data
   * object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 9, 2006: Created (LEM)
   */
  public boolean SetVectorValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        java.util.Vector p_oData,
                                        String[] p_sChildXMLTags,
                                        boolean[] p_bAppliesTo,
                                        org.xml.sax.Attributes
                                        oParentAttributes,
                                        org.xml.sax.Attributes[] p_oAttributes) throws
      ModelException {

      if (sXMLTag.equals("st_stmPropDieMedDmg")) {
        int i;
        //Old proportion of medium damaged trees that die - transform
        //into equivalent parameters for new logit function
        sXMLTag = "st_stmMedDmgSurvProbA";
        for (i = 0; i < p_sChildXMLTags.length; i++) {
          if (p_sChildXMLTags[i].equals("st_spdmdVal")) {
            p_sChildXMLTags[i] = "st_smdspaVal";
          }
        }

        //Transform proportion values into "a" values - these will be set
        //at the end of this function
        Float fA;
        for (i = 0; i < p_oData.size(); i++) {
          fA = new Float(p_oData.elementAt(i).toString());
          if (fA.floatValue() == 0) fA = new Float(10);
          else fA = new Float(java.lang.Math.log(1 / ((1 / (1 - fA.floatValue())) - 1)));
          p_oData.remove(i);
          p_oData.add(i, fA);
        }

        //Now set "b" values to 0
        Float[] p_fB = new Float[p_oData.size()];
        for (i = 0; i < p_fB.length; i++) p_fB[i] = new Float(0);
        SetVectorValues(mp_fStmMedDmgSurvProbB, p_fB, p_bAppliesTo);

        //Enable the storm killer behavior and give it the same
        //species/type combos as the storm damage applier
        Behavior oStormKiller = GetBehaviorByKey("Storm Killer"),
                 oDamageApplier = GetBehaviorByKey("Storm Damage Applier");
        SpeciesTypeCombo oCombo;
        oStormKiller.m_bIsEnabled = true;
        oStormKiller.ClearSpeciesTypeCombos();
        for (i = 0; i < oDamageApplier.GetNumberOfCombos(); i++) {
          oCombo = (SpeciesTypeCombo) oDamageApplier.GetSpeciesTypeCombo(i).clone();
          oStormKiller.AddSpeciesTypeCombo(oCombo);
        }
      }
      else if (sXMLTag.equals("st_stmPropDieFullDmg")) {
        int i;
        //Old proportion of heavily damaged trees that die - transform
        //into equivalent parameters for new logit function
        sXMLTag = "st_stmFullDmgSurvProbA";
        for (i = 0; i < p_sChildXMLTags.length; i++) {
          if (p_sChildXMLTags[i].equals("st_spdfdVal")) {
            p_sChildXMLTags[i] = "st_sfdspaVal";
          }
        }

        //Transform proportion values into "a" values - these will be set
        //at the end of this function
        Float fA;
        for (i = 0; i < p_oData.size(); i++) {
          fA = new Float(p_oData.elementAt(i).toString());
          if (fA.floatValue() == 0) fA = new Float(10);
          else fA = new Float(java.lang.Math.log(1 / ((1 / (1 - fA.floatValue())) - 1)));
          p_oData.remove(i);
          p_oData.add(i, fA);
        }

        //Now set "b" values to 0
        Float[] p_fB = new Float[p_oData.size()];
        for (i = 0; i < p_fB.length; i++) p_fB[i] = new Float(0);
        SetVectorValues(mp_fStmFullDmgSurvProbB, p_fB, p_bAppliesTo);

        //The storm killer behavior should be enabled when the medium damage
        //tag comes through
      }

    return super.SetVectorValueByXMLTag(sXMLTag, sXMLParentTag, p_oData,
                                        p_sChildXMLTags, p_bAppliesTo,
                                        oParentAttributes, p_oAttributes);
  }

  /**
   * If the harvest interface behavior is being used, this will check to see
   * if it needs to add new data members to the name.  After changing the name
   * and writing the behaviors, the name will be changed back.
   * @param out BufferedWriter File to write to.
   * @param oPop TreePopulation Tree population.
   * @throws ModelException Passing through from called functions.
   */
  void WriteBehaviorNodes(BufferedWriter out, TreePopulation oPop) throws
      ModelException {
    Behavior oBeh = GetBehaviorByKey("Harvest Interface");
    boolean bDoIt = oBeh.m_bIsEnabled && mp_sHarvIntNewTreeDataMembers.size() > 0;
    String sXMLName = "";
    if (bDoIt) {
      sXMLName = oBeh.m_sParFileTag;
      for (int i = 0; i < mp_sHarvIntNewTreeDataMembers.size(); i++) {
        oBeh.m_sParFileTag += " (" +
            (String)mp_sHarvIntNewTreeDataMembers.elementAt(i) + ")";
      }
    }

    super.WriteBehaviorNodes(out, oPop);

    if (bDoIt) oBeh.m_sParFileTag = sXMLName;
  }

  /**
   * Overwritten to handle the potentially-funky harvest interface behavior.
   * @param sXMLTag String The XML tag for which to find a behavior.
   * @return Behavior Behavior for the XML tag, or NULL if none of the
   * behaviors has that tag.
   */
  public Behavior GetBehaviorByXMLTag(String sXMLTag) {
    if (sXMLTag.startsWith("Harvest Interface")) {
      mp_sHarvIntNewTreeDataMembers.clear();
      String sDataMember;
      int iStart = sXMLTag.indexOf('('), iEnd;
      if (iStart >= 0) {
        while (iStart >= 0) {
          iEnd = sXMLTag.indexOf(')', iStart);
          if (iEnd < 0) {
            //fail quietly - we don't have a choice
            break;
          }
          iStart++;
          sDataMember = sXMLTag.substring(iStart, iEnd);
          mp_sHarvIntNewTreeDataMembers.add(sDataMember);
          iStart = sXMLTag.indexOf('(', iEnd);
        }
        sXMLTag = "Harvest Interface";
      }
    }
    return super.GetBehaviorByXMLTag(sXMLTag);
  }

}
