package javawrapper;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.util.Vector;

/**
 * This is a window allowing setup of the Harvest Interface behavior.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 * <br>Edit history:
 * <br>-------------------
 * <br> September 29, 2006: Created (LEM)
 */

public class HarvestInterfaceSetup
    extends JDialog
    implements java.awt.event.ActionListener {

  /** Main window. */
  protected MainWindow m_oWindow;

  /** GUIManager object.*/
  protected GUIManager m_oManager;

  /** Copy of the behavior, so we can cancel edits if the user desires */
  protected Behavior m_oBehaviorCopy;

  /** Field for capturing the executable's path and filename*/
  protected JTextField m_jExecutablePathEdit = new JTextField();

  /** Field for capturing the filename that SORTIE is to write*/
  protected JTextField m_jSORTIEOutFileEdit = new JTextField();

  /** Field for capturing the executable's harvest reply file*/
  protected JTextField m_jExecutableHarvestOutFileEdit = new JTextField();

  /** Field for capturing the executable's update reply file*/
  protected JTextField m_jExecutableUpdateOutFileEdit = new JTextField();

  /** Field for capturing the batch parameters file*/
  protected JTextField m_jBatchParamsFileEdit = new JTextField();

  /** Field for capturing the file for a single run of batch parameters*/
  protected JTextField m_jBatchSingleRunParamsFileEdit = new JTextField();

  /** Field for entering new tree data members */
  protected JTextField m_jNewTreeDataMemberEdit = new JTextField();

  /** Field for capturing argument strings to pass to the executable */
  protected JTextField m_jExecutableArgs = new JTextField();

  /** Field for capturing the harvest period */
  protected JTextField m_jHarvestPeriod = new JTextField();

  /** The list displaying the assigned species/type combos for this
   * behavior*/
  protected JList m_jAssignedCombosList;

  /** The list displaying the new tree data members for this behavior */
  protected JList m_jNewTreeDataMembersList;

  /** The list displaying the file columns */
  protected JList m_jFileColumnsList;

  /**The list of species*/
  protected JList m_jSpecies;

  /**The list of types*/
  protected JList m_jTypes;

  /**The assigned species/type combos for this behavior*/
  protected DefaultListModel m_jAssignedCombosListModel;

  /** The new tree data members for this behavior */
  protected DefaultListModel m_jNewTreeDataMembersListModel;

  /** The file columns for the harvest files */
  protected DefaultListModel m_jFileColumnsListModel;

  /** List of existing data members, so we can make sure any added have unique
   * names */
  protected Vector mp_sAllExistingDataMembers = new Vector(0);

  /** The ID of the help file for this window */
  private String m_sHelpID = "windows.edit_harvest_interface_window";

  /**
   * Constructor.
   * @param oParent JFrame Parent frame
   * @param oManager GUIManager GUIManager object
   * @throws ModelException Passing through from called functions.
   */
  public HarvestInterfaceSetup(JFrame oParent, GUIManager oManager) throws
      ModelException {
    super(oParent, "Edit Harvest Interface", true);
    m_oManager = oManager;
    m_oWindow = (MainWindow) oParent;

    //Make a copy of the harvest interface behavior
    m_oBehaviorCopy = (Behavior) m_oManager.GetDisturbanceBehaviors().
        GetBehaviorByKey("Harvest Interface").clone();

    LoadData();
    CreateGUI();
  }

  /**
   * Extracts data from the harvest interface behavior, making it ready for
   * display
   * @throws ModelException Passing through from other called functions.
   */
  protected void LoadData() throws ModelException {
    TreePopulation oPop = m_oManager.GetTreePopulation();
    DisturbanceBehaviors oDisturbance = m_oManager.GetDisturbanceBehaviors();
    int i, j, k, m;

    //**************************
    //File names and basic data
    //**************************
    m_jExecutablePathEdit.setText(oDisturbance.m_sHarvIntExecutable.GetValue());
    m_jBatchParamsFileEdit.setText(oDisturbance.m_sHarvIntBatchParamsFile.
                                   GetValue());
    m_jSORTIEOutFileEdit.setText(oDisturbance.m_sHarvIntSORTIEOutFile.GetValue());
    m_jBatchSingleRunParamsFileEdit.setText(oDisturbance.
                                            m_sHarvIntBatchSingleRunParamsFile.
                                            GetValue());
    m_jExecutableHarvestOutFileEdit.setText(oDisturbance.
                                            m_sHarvIntExecHarvestOutFile.
                                            GetValue());
    m_jExecutableArgs.setText(oDisturbance.m_sHarvIntExecArgs.GetValue());
    m_jHarvestPeriod.setText(String.valueOf(oDisturbance.
                                            m_iHarvIntHarvestPeriod.GetValue()));
    m_jExecutableUpdateOutFileEdit.setText(oDisturbance.
                                           m_sHarvIntExecUpdateOutFile.GetValue());

    //**************************
    //Species/type combos to which this behavior applies
    //**************************
    m_jAssignedCombosListModel = new DefaultListModel();
    for (i = 0; i < m_oBehaviorCopy.GetNumberOfCombos(); i++) {
      m_jAssignedCombosListModel.addElement(
          new ComboDisplay(oPop,
                           m_oBehaviorCopy.GetSpeciesTypeCombo(i).GetSpecies(),
                           m_oBehaviorCopy.GetSpeciesTypeCombo(i).GetType()));
    }

    //**************************
    //File columns
    //**************************
    m_jFileColumnsListModel = new DefaultListModel();
    //Add the default column list
    m_jFileColumnsListModel.addElement("X");
    m_jFileColumnsListModel.addElement("Y");
    m_jFileColumnsListModel.addElement("Species");
    m_jFileColumnsListModel.addElement("Type");
    m_jFileColumnsListModel.addElement("Diam");
    m_jFileColumnsListModel.addElement("Height");
    //Add a dashed line to separate mandatory from additional columns
    m_jFileColumnsListModel.addElement("-------");

    for (i = 0; i < oDisturbance.mp_sHarvIntFileColumns.GetValue().size(); i++) {
      m_jFileColumnsListModel.addElement(oDisturbance.mp_sHarvIntFileColumns.
                                         GetValue().elementAt(i));
    }

    //**************************
    //New data members being created by this behavior
    //**************************
    m_jNewTreeDataMembersListModel = new DefaultListModel();
    for (i = 0; i < oDisturbance.mp_sHarvIntNewTreeDataMembers.size(); i++) {
      m_jNewTreeDataMembersListModel.addElement(oDisturbance.
                                                mp_sHarvIntNewTreeDataMembers.
                                                elementAt(i));
    }

    //**************************
    //All existing data members defined in the behavior, whether being used
    //in this run or not
    //**************************
    //Add those data members defined in the tree population
    mp_sAllExistingDataMembers.add("X");
    mp_sAllExistingDataMembers.add("Y");
    mp_sAllExistingDataMembers.add("Diam10");
    mp_sAllExistingDataMembers.add("Height");
    mp_sAllExistingDataMembers.add("DBH");
    mp_sAllExistingDataMembers.add("Age");

    BehaviorTypeBase[] p_oGroups = m_oManager.GetAllBehaviorGroups();
    Behavior[] p_oBehaviors;
    DataMember oDataMember;
    String sDataMemberName;
    boolean bFound;
    for (i = 0; i < p_oGroups.length; i++) {
      p_oBehaviors = p_oGroups[i].GetAllBehaviors();
      for (j = 0; j < p_oBehaviors.length; j++) {
        for (k = 0; k < p_oBehaviors[j].mp_oNewTreeDataMembers.size(); k++) {
          oDataMember = (DataMember) p_oBehaviors[j].mp_oNewTreeDataMembers.elementAt(k);
          sDataMemberName = oDataMember.GetCodeName();
          bFound = false;
          for (m = 0; m < mp_sAllExistingDataMembers.size(); m++) {
            if (sDataMemberName.equals((String)mp_sAllExistingDataMembers.elementAt(m))) {
              bFound = true;
              break;
            }
          }
          if (false == bFound) {
            mp_sAllExistingDataMembers.add(sDataMemberName);
          }
        }
      }
    }

  }

  /**
   * Controls actions for this window.
   * @param e ActionEvent.
   */
  public void actionPerformed(java.awt.event.ActionEvent e) {
    try {
      String sCommand = e.getActionCommand();
      if (sCommand.equals("Cancel")) {
        this.setVisible(false);
        this.dispose();
      }
      else if (sCommand.equals("OK")) {
        if (OK()) {
          this.setVisible(false);
          this.dispose();
        }
      }
      else if (sCommand.equals("Edit_Combos")) {
        new DisplayBehaviorComboEdit(this,
                                     m_oBehaviorCopy,
                                     m_oManager.GetTreePopulation(),
                                     m_oManager.GetHelpBroker());

        //Update the behavior's combo list - doing it always is easier than
        //checking for whether or not it changed
        int i;
        m_jAssignedCombosListModel.clear();
        TreePopulation oPop = m_oManager.GetTreePopulation();
        for (i = 0; i < m_oBehaviorCopy.GetNumberOfCombos(); i++) {
          m_jAssignedCombosListModel.addElement(
              new ComboDisplay(oPop,
                               m_oBehaviorCopy.GetSpeciesTypeCombo(i).
                               GetSpecies(),
                               m_oBehaviorCopy.GetSpeciesTypeCombo(i).GetType()));
        }
      }
      else if (sCommand.equals("Browse_BatchParamsFile")) {
        Browse(m_jBatchParamsFileEdit);
      }
      else if (sCommand.equals("Browse_SORTIEOutFile")) {
        Browse(m_jSORTIEOutFileEdit);
      }
      else if (sCommand.equals("Browse_BatchParamsSingleRunFile")) {
        Browse(m_jBatchSingleRunParamsFileEdit);
      }
      else if (sCommand.equals("Browse_ExecHarvestOutFile")) {
        Browse(m_jExecutableHarvestOutFileEdit);
      }
      else if (sCommand.equals("Browse_ExecUpdateOutFile")) {
        Browse(m_jExecutableUpdateOutFileEdit);
      }
      else if (sCommand.equals("Browse_Executable")) {
        Browse(m_jExecutablePathEdit);
      }
      else if (sCommand.equals("Add_Column")) {
        if (m_jAssignedCombosListModel.size() == 0) {
          JOptionPane.showMessageDialog(this,
              "Please choose trees for this behavior to apply to.");
          return;
        }
        new FileColumnsChooser(this);
      }
      else if (sCommand.equals("Remove_Column")) {
        RemoveColumn();
      }
      else if (sCommand.equals("Column_Up")) {
        ColumnUp();
      }
      else if (sCommand.equals("Column_Down")) {
        ColumnDown();
      }
      else if (sCommand.equals("Add_Member")) {
        AddDataMember();
      }
      else if (sCommand.equals("Remove_Member")) {
        RemoveDataMember();
      }
    }
    catch (ModelException oErr) {
      ErrorGUI oHandler = new ErrorGUI(this);
      oHandler.WriteErrorMessage(oErr);
    }
  }

  /**
   * Creates the window.
   */
  private void CreateGUI() {
    int iWidth = 300; //preferred width of file paths and scrollers

    //******************************************
     // At the top of the window: basic controls, in a gridded pattern
     //******************************************
    JPanel jNorthPanel = new JPanel(new GridLayout(0, 2));

    //*
     //Row 1, col 1: Components for collecting the path to the executable
     //*
    JPanel jCellPanel = new JPanel();
    jCellPanel.setLayout(new javax.swing.BoxLayout(jCellPanel,
        javax.swing.BoxLayout.PAGE_AXIS));
    JLabel jLabel = new JLabel("Path and filename of the executable:");
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jLabel.setFont(new ModelFont());
    jCellPanel.add(jLabel);
    //Text box and browse button for collecting the path to the
    //executable, packaged together in a panel so they'll be side-by-side
    JPanel jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.
        LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jExecutablePathEdit.setFont(new ModelFont());
    m_jExecutablePathEdit.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jExecutablePathEdit.setPreferredSize(new Dimension(iWidth, 25));
    jTempPanel.add(m_jExecutablePathEdit);
    JButton jButton = new JButton("Browse");
    jButton.setFont(new ModelFont());
    jButton.setAlignmentX(Component.LEFT_ALIGNMENT);
    jButton.addActionListener(this);
    jButton.setActionCommand("Browse_Executable");
    jTempPanel.add(jButton);
    jCellPanel.add(jTempPanel);
    jNorthPanel.add(jCellPanel);
    //Save the button size so we can make all buttons this size
    Dimension jButtonSize = jButton.getPreferredSize();

    //*
     //Row 1, col 2: Components for collecting the path to the batch parameters
     //file
     //*
    jCellPanel = new JPanel();
    jCellPanel.setLayout(new javax.swing.BoxLayout(jCellPanel,
        javax.swing.BoxLayout.PAGE_AXIS));
    jLabel = new JLabel("Parameters file for batch run (optional):");
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jLabel.setFont(new ModelFont());
    jCellPanel.add(jLabel);
    //Text box and browse button for collecting the path to the
    //executable harvest out file, packaged together in a panel so they'll be
    //side-by-side
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jBatchParamsFileEdit.setFont(new ModelFont());
    m_jBatchParamsFileEdit.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jBatchParamsFileEdit.setPreferredSize(new Dimension(iWidth, 25));
    jTempPanel.add(m_jBatchParamsFileEdit);
    jButton = new JButton("Browse");
    jButton.setFont(new ModelFont());
    jButton.setAlignmentX(Component.LEFT_ALIGNMENT);
    jButton.addActionListener(this);
    jButton.setActionCommand("Browse_BatchParamsFile");
    jTempPanel.add(jButton);
    jCellPanel.add(jTempPanel);
    jNorthPanel.add(jCellPanel);

    //*
     //Row 2, col 1: Components for collecting the path to the SORTIE out file
     //*
    jCellPanel = new JPanel();
    jCellPanel.setLayout(new javax.swing.BoxLayout(jCellPanel,
        javax.swing.BoxLayout.PAGE_AXIS));
    jLabel = new JLabel("Tree file that SORTIE will write:");
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jLabel.setFont(new ModelFont());
    jCellPanel.add(jLabel);
    //Text box and browse button for collecting the path to the
    //SORTIE out file, packaged together in a panel so they'll be side-by-side
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jSORTIEOutFileEdit.setFont(new ModelFont());
    m_jSORTIEOutFileEdit.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jSORTIEOutFileEdit.setPreferredSize(new Dimension(iWidth, 25));
    jTempPanel.add(m_jSORTIEOutFileEdit);
    jButton = new JButton("Browse");
    jButton.setFont(new ModelFont());
    jButton.setAlignmentX(Component.LEFT_ALIGNMENT);
    jButton.addActionListener(this);
    jButton.setActionCommand("Browse_SORTIEOutFile");
    jTempPanel.add(jButton);
    jCellPanel.add(jTempPanel);
    jNorthPanel.add(jCellPanel);

    //*
     //Row 2, col 2: Components for collecting the path to the single-run batch
     //parameters file to write
     //*
    jCellPanel = new JPanel();
    jCellPanel.setLayout(new javax.swing.BoxLayout(jCellPanel,
        javax.swing.BoxLayout.PAGE_AXIS));
    jLabel = new JLabel("Single-run parameters file for batch run (optional):");
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jLabel.setFont(new ModelFont());
    jCellPanel.add(jLabel);
    //Text box and browse button for collecting the path to the
    //executable harvest out file, packaged together in a panel so they'll be
    //side-by-side
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jBatchSingleRunParamsFileEdit.setFont(new ModelFont());
    m_jBatchSingleRunParamsFileEdit.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jBatchSingleRunParamsFileEdit.setPreferredSize(new Dimension(iWidth, 25));
    jTempPanel.add(m_jBatchSingleRunParamsFileEdit);
    jButton = new JButton("Browse");
    jButton.setFont(new ModelFont());
    jButton.setAlignmentX(Component.LEFT_ALIGNMENT);
    jButton.addActionListener(this);
    jButton.setActionCommand("Browse_BatchParamsSingleRunFile");
    jTempPanel.add(jButton);
    jCellPanel.add(jTempPanel);
    jNorthPanel.add(jCellPanel);

    //*
     //Row 3, col 1: Components for collecting the path to the executable harvest
     //out file
     //*
    jCellPanel = new JPanel();
    jCellPanel.setLayout(new javax.swing.BoxLayout(jCellPanel,
        javax.swing.BoxLayout.PAGE_AXIS));
    jLabel = new JLabel("Tree harvest file that the executable will write:");
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jLabel.setFont(new ModelFont());
    jCellPanel.add(jLabel);
    //Text box and browse button for collecting the path to the
    //executable harvest out file, packaged together in a panel so they'll be
    //side-by-side
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jExecutableHarvestOutFileEdit.setFont(new ModelFont());
    m_jExecutableHarvestOutFileEdit.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jExecutableHarvestOutFileEdit.setPreferredSize(new Dimension(iWidth, 25));
    jTempPanel.add(m_jExecutableHarvestOutFileEdit);
    jButton = new JButton("Browse");
    jButton.setFont(new ModelFont());
    jButton.setAlignmentX(Component.LEFT_ALIGNMENT);
    jButton.addActionListener(this);
    jButton.setActionCommand("Browse_ExecHarvestOutFile");
    jTempPanel.add(jButton);
    jCellPanel.add(jTempPanel);
    jNorthPanel.add(jCellPanel);

    //*
     //Row 3, col 2: Components for collecting arguments to pass to the
     //executable
     //*
    jCellPanel = new JPanel();
    jCellPanel.setLayout(new javax.swing.BoxLayout(jCellPanel,
        javax.swing.BoxLayout.PAGE_AXIS));
    jLabel = new JLabel("Arguments to pass to the executable (optional):");
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jLabel.setFont(new ModelFont());
    jCellPanel.add(jLabel);
    //Text box for collecting arguments to pass to the executable - doesn't
    //need a button but we'll set it up like col 1 so they line up correctly
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jExecutableArgs.setFont(new ModelFont());
    m_jExecutableArgs.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jExecutableArgs.setPreferredSize(new Dimension(iWidth, 25));
    jTempPanel.add(m_jExecutableArgs);
    jCellPanel.add(jTempPanel);
    jNorthPanel.add(jCellPanel);

    //Row 4, col 1: Components for collecting the harvest period
    jCellPanel = new JPanel();
    jCellPanel.setLayout(new javax.swing.BoxLayout(jCellPanel,
        javax.swing.BoxLayout.PAGE_AXIS));
    jLabel = new JLabel("How often to harvest, in years:");
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jLabel.setFont(new ModelFont());
    jCellPanel.add(jLabel);
    //Text box for collecting the harvest period - doesn't need a
    //button but we'll set it up like col 1 so they line up correctly
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jHarvestPeriod.setFont(new ModelFont());
    m_jHarvestPeriod.setPreferredSize(new Dimension(50, 25));
    jTempPanel.add(m_jHarvestPeriod);
    jCellPanel.add(jTempPanel);
    jNorthPanel.add(jCellPanel);

    //******************************************
     // What's in the center-left of the window - scroll panes
     //******************************************
    JPanel jCenterWestPanel = new JPanel();
    jCenterWestPanel.setLayout(new javax.swing.BoxLayout(jCenterWestPanel,
        javax.swing.BoxLayout.PAGE_AXIS));

    //*
     //The trees to which this behavior applies
     //*
    jLabel = new JLabel("Behavior currently assigned to:");
    jLabel.setFont(new ModelFont());
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jCenterWestPanel.add(jLabel);

    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.
        LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jAssignedCombosList = new JList(m_jAssignedCombosListModel);
    m_jAssignedCombosList.setFont(new ModelFont());
    m_jAssignedCombosList.setVisibleRowCount(4);
    JScrollPane jScroller = new JScrollPane(m_jAssignedCombosList);
    jScroller.setPreferredSize(new Dimension(iWidth,
                                             (int) jScroller.getPreferredSize().
                                             getHeight()));
    jTempPanel.add(jScroller);
    //Edit button for combos
    jButton = new JButton("Edit");
    jButton.setFont(new ModelFont());
    jButton.setToolTipText(
        "Edit the tree species / types for Harvest Interface");
    jButton.setActionCommand("Edit_Combos");
    jButton.addActionListener(this);
    jButton.setPreferredSize(jButtonSize);
    jTempPanel.add(jButton);
    jCenterWestPanel.add(jTempPanel);

    //*
     // File columns
     //*
    jLabel = new JLabel("File columns:");
    jLabel.setFont(new ModelFont());
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jCenterWestPanel.add(jLabel);

    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.
        LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jFileColumnsList = new JList(m_jFileColumnsListModel);
    m_jFileColumnsList.setFont(new ModelFont());
    jScroller = new JScrollPane(m_jFileColumnsList);
    m_jFileColumnsList.setVisibleRowCount(5);
    jScroller.setPreferredSize(new Dimension(iWidth,
                                             (int) jScroller.getPreferredSize().
                                             getHeight()));
    jTempPanel.add(jScroller);

    //Panel containing Add/Remove buttons for the file columns list
    JPanel jButtonPanel = new JPanel();
    jButtonPanel.setLayout(new java.awt.GridLayout(0, 1));

    jButton = new JButton("Add");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Add_Column");
    jButton.addActionListener(this);
    jButtonPanel.add(jButton);

    jButton = new JButton("Remove");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Remove_Column");
    jButton.addActionListener(this);
    jButtonPanel.add(jButton);
    jTempPanel.add(jButtonPanel);
    jCenterWestPanel.add(jTempPanel);

    jButton = new JButton("Up");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Column_Up");
    jButton.addActionListener(this);
    jButtonPanel.add(jButton);
    jTempPanel.add(jButtonPanel);
    jCenterWestPanel.add(jTempPanel);

    jButton = new JButton("Down");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Column_Down");
    jButton.addActionListener(this);
    jButtonPanel.add(jButton);
    jTempPanel.add(jButtonPanel);
    jCenterWestPanel.add(jTempPanel);



    //******************************************
     // Panel for optional tree updating - center east
     //******************************************
    JPanel jCenterEastPanel = new JPanel();
    jCenterEastPanel.setLayout(new javax.swing.BoxLayout(jCenterEastPanel,
        javax.swing.BoxLayout.PAGE_AXIS));
    //The title label
    jLabel = new JLabel("Tree Updating (optional)...");
    jLabel.setFont(new ModelFont(java.awt.Font.BOLD, 2));
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jCenterEastPanel.add(jLabel);
    jCenterEastPanel.add(javax.swing.Box.createRigidArea(new Dimension(0, 10)));

    //The grouping for collecting the path to the executable update out file
    jLabel = new JLabel("Tree update file that the executable will write:");
    jLabel.setFont(new ModelFont());
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jCenterEastPanel.add(jLabel);

    //Package the detailed output edit box and the browse button in a panel so
    //they'll be side-by-side
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.
        LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jExecutableUpdateOutFileEdit.setFont(new ModelFont());
    m_jExecutableUpdateOutFileEdit.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jExecutableUpdateOutFileEdit.setPreferredSize(new Dimension(iWidth, 25));
    jTempPanel.add(m_jExecutableUpdateOutFileEdit);

    jButton = new JButton("Browse");
    jButton.setFont(new ModelFont());
    jButton.setAlignmentX(Component.LEFT_ALIGNMENT);
    jButton.addActionListener(this);
    jButton.setActionCommand("Browse_ExecUpdateOutFile");
    jTempPanel.add(jButton);
    jCenterEastPanel.add(jTempPanel);
    jCenterEastPanel.add(javax.swing.Box.createRigidArea(new Dimension(0, 5)));

    // New data members
    jLabel = new JLabel("New tree data members to add:");
    jLabel.setFont(new ModelFont());
    jLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
    jCenterEastPanel.add(jLabel);

    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.
        LEFT));
    jTempPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    m_jNewTreeDataMembersList = new JList(m_jNewTreeDataMembersListModel);
    m_jNewTreeDataMembersList.setFont(new ModelFont());
    m_jNewTreeDataMembersList.setVisibleRowCount(5);
    jScroller = new JScrollPane(m_jNewTreeDataMembersList);
    jScroller.setPreferredSize(new Dimension(iWidth,
                                             (int) jScroller.getPreferredSize().
                                             getHeight()));
    jTempPanel.add(jScroller);

    //Panel containing Add/Remove buttons for the new data members list
    jButtonPanel = new JPanel();
    jButtonPanel.setLayout(new java.awt.GridLayout(0, 1));
    jLabel = new JLabel("New member:");
    jLabel.setFont(new ModelFont());
    jButtonPanel.add(jLabel);
    m_jNewTreeDataMemberEdit = new JTextField();
    m_jNewTreeDataMemberEdit.setFont(new ModelFont());
    jButtonPanel.add(m_jNewTreeDataMemberEdit);

    jButton = new JButton("Add");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Add_Member");
    jButton.addActionListener(this);
    jButtonPanel.add(jButton);

    jButton = new JButton("Remove");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Remove_Member");
    jButton.addActionListener(this);
    jButtonPanel.add(jButton);
    jTempPanel.add(jButtonPanel);
    jCenterEastPanel.add(jTempPanel);

    //****************************
     //Put it all together
     //****************************
    JPanel jContentPane = (JPanel)this.getContentPane();
    jContentPane.setLayout(new BorderLayout());
    jContentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 0, 5));
    jContentPane.add(jNorthPanel, BorderLayout.NORTH);
    JPanel jCenterPanel = new JPanel(new GridLayout(1, 2));
    jCenterPanel.add(jCenterWestPanel);
    jCenterPanel.add(jCenterEastPanel);
    jContentPane.add(jCenterPanel, BorderLayout.CENTER);
    jContentPane.add(new OKCancelButtonPanel(this,
                                             m_oManager.GetHelpBroker(),
                                             m_sHelpID, true),
                     BorderLayout.SOUTH);
    this.setContentPane(jContentPane);
  }

  /**
   * Takes care of browsing
   * @param jText JTextField The file field to browse for
   */
  private void Browse(JTextField jText) {
    ModelFileChooser jChooser = new ModelFileChooser();

    //If there's a value for the detailed output file, navigate to that directory
    File oDir = new File(jText.getText());
    if (oDir.getParentFile() != null && oDir.getParentFile().exists()) {
      jChooser.setCurrentDirectory(oDir.getParentFile());
    }
    jChooser.setSelectedFile(new File(oDir.getName()));
    int returnVal = jChooser.showOpenDialog(this);
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      //User chose a file - put the filename in the edit box
      File oFile = jChooser.getSelectedFile();
      String sFileName = oFile.getAbsolutePath();
      jText.setText(sFileName);
    }
  }

  /**
   * Removes file columns from the list to add.
   * @throws ModelException if a column is not selected, or if the user tries
   * to remove a default column from the list.
   */
  protected void RemoveColumn() throws ModelException {
    Object[] p_oSelectedObjects = m_jFileColumnsList.getSelectedValues();
    int[] p_iSelected = m_jFileColumnsList.getSelectedIndices();
    int i, iDashedIndex = 0;

    //None selected?  Exit.
    if (p_iSelected.length == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "No columns selected.  Nothing to remove."));
    }

    //Where is the dashed line?  Anything before that index cannot be removed.
    for (i = 0; i < m_jFileColumnsListModel.size(); i++) {
      if (((String)m_jFileColumnsListModel.elementAt(i)).startsWith("--")) {
        iDashedIndex = i;
        break;
      }
    }

    //See if any of the selected indexes are default columns
    for (i = 0; i < p_iSelected.length; i++) {
      if (p_iSelected[i] <= iDashedIndex) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The default columns cannot be removed."));
      }
    }

    //Get each save string
    for (i = 0; i < p_iSelected.length; i++) {
      m_jFileColumnsListModel.removeElement(p_oSelectedObjects[i]);
    }
  }

  /**
   * Moves a selected column up one position in the list.
   * @throws ModelException if the user has tried to disturb the default
   * columns.
   */
  protected void ColumnUp() throws ModelException {
    //Move the selected behavior up in the list - but it can't
    //go to a smaller group index
    int iSelectedIndex = m_jFileColumnsList.getSelectedIndex(), //selection
        iDashedIndex = 0, //index of the dashed line above which this cannot move
        iNewIndex = iSelectedIndex - 1, //proposed new position of item
        i;

    //Where is the dashed line?
    for (i = 0; i < m_jFileColumnsListModel.size(); i++) {
      if (((String)m_jFileColumnsListModel.elementAt(i)).startsWith("--")) {
        iDashedIndex = i;
        break;
      }
    }

    //If nothing selected or the move would affect default columns,
    //warn and exit
    if (-1 == iSelectedIndex) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "Nothing selected to move."));
    }
    if (iSelectedIndex <= iDashedIndex || iNewIndex <= iDashedIndex) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The default columns cannot be changed."));
    }

    //Swap the object from old position to new
    Object oItem = m_jFileColumnsListModel.getElementAt(iSelectedIndex);
    m_jFileColumnsListModel.insertElementAt(oItem, iNewIndex);
    m_jFileColumnsListModel.removeElementAt(iSelectedIndex + 1);

    //Keep the same item selected
    m_jFileColumnsList.setSelectedIndex(iNewIndex);
  }

  /**
   * Moves a selected column down one position in the list.
   * @throws ModelException if the request is to move a default column.
   */
  protected void ColumnDown() throws ModelException {
    //Move the selected behavior up in the list - but it can't
    //go to a smaller group index
    int iSelectedIndex = m_jFileColumnsList.getSelectedIndex(), //selection
        iDashedIndex = 0, //index of the dashed line above which this cannot move
        iNewIndex = iSelectedIndex + 2, //proposed new position of item
        i;

    //Where is the dashed line?
    for (i = 0; i < m_jFileColumnsListModel.size(); i++) {
      if (((String)m_jFileColumnsListModel.elementAt(i)).startsWith("--")) {
        iDashedIndex = i;
        break;
      }
    }

    //If nothing selected or the move would affect the default columns, warn
    //and exit
    if (-1 == iSelectedIndex || (m_jFileColumnsListModel.getSize() - 1) == iSelectedIndex) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "Nothing selected to move."));
    }
    if (iSelectedIndex <= iDashedIndex) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The default columns cannot be changed."));
    }

    //Swap the object from old position to new
    Object oItem = m_jFileColumnsListModel.getElementAt(iSelectedIndex);
    m_jFileColumnsListModel.insertElementAt(oItem, iNewIndex);
    m_jFileColumnsListModel.removeElementAt(iSelectedIndex);

    //Keep the same item selected
    m_jFileColumnsList.setSelectedIndex(iNewIndex - 1);
  }

  /**
   * Adds a new data member, to be owned by this behavior.
   * @throws ModelException if:
   * <ul>
   * <li>The data member is longer than 9 characters</li>
   * <li>It has the same name as an existing data member</li>
   * <li>It has parentheses in the name</li>
   * </ul>
   */
  protected void AddDataMember() throws ModelException {
    String sNewMember = m_jNewTreeDataMemberEdit.getText().trim();
    int iMaxLabelSize = 9, i;

    if (sNewMember.length() == 0) {
      m_jNewTreeDataMemberEdit.setText("");
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "No new data member name specified."));
    }

    //Make sure the name isn't too long
    if (sNewMember.length() > iMaxLabelSize) {
      m_jNewTreeDataMemberEdit.setText("");
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                               "Data member names cannot be longer than " +
                               iMaxLabelSize + " characters."));
    }

    //Make sure the name doesn't already exist
    for (i = 0; i < mp_sAllExistingDataMembers.size(); i++) {
      if (sNewMember.equals((String)mp_sAllExistingDataMembers.elementAt(i))) {
        m_jNewTreeDataMemberEdit.setText("");
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The name \"" +
           sNewMember + "\" is already being used for a data member name in SORTIE."));
      }
    }

    //Make sure the name hasn't already been specified as a new member to
    //add
    for (i = 0; i < m_jNewTreeDataMembersListModel.size(); i++) {
      if (sNewMember.equals((String)m_jNewTreeDataMembersListModel.elementAt(i))) {
        m_jNewTreeDataMemberEdit.setText("");
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "This data member is already on the list of new data members."));
      }
    }

    //Make sure the name does not contain any parentheses
    if (sNewMember.indexOf('(') >= 0 ||
        sNewMember.indexOf(')') >= 0) {
      m_jNewTreeDataMemberEdit.setText("");
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The data member name cannot contain parentheses."));
    }

    //Add it
    m_jNewTreeDataMembersListModel.addElement(sNewMember);
    m_jNewTreeDataMemberEdit.setText("");
  }

  /**
   * Removes a new data member.
   * @throws ModelException if there is not a data member selected.
   */
  protected void RemoveDataMember() throws ModelException {
    Object[] p_oSelected = m_jNewTreeDataMembersList.getSelectedValues();
    int i;

    //None selected?  Exit.
    if (p_oSelected.length == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "No data members selected.  Nothing to remove."));
    }

    //Get each save string
    for (i = 0; i < p_oSelected.length; i++) {
      m_jNewTreeDataMembersListModel.removeElement(p_oSelected[i]);
    }
  }

  /**
   * What happens when the OK button is clicked.  The data is validated and
   * passed to the DisturbanceBehaviors object as needed.
   * @return true if everything is correct; false if the process has been
   * interrupted and the window should not close.
   * @throws ModelException if:
   * <ul>
   * <li>There are no species/type combos assigned</li>
   * <li>The path is missing to the executable, the harvest out file, or the
   * harvest in file</li>
   * <li>The executable can't be found</li>
   * <li>A warning is issued if there is an update file listed but no new
   * tree data members</li>
   * <li>The harvest period is not specified, is not a number, or is
   * negative</li>
   * <li>Either the batch parameters file or the single run batch parameters
   * file is present without the other</li>
   * <li>The batch parameters file is specified but does not exist</li>
   * </ul>
   */
  protected boolean OK() throws ModelException {
    DisturbanceBehaviors oDisturbance = m_oManager.GetDisturbanceBehaviors();
    String sData;
    int i;

    //*******************
    // Data validation
    //*******************
    //Make sure the behavior has been assigned to trees
    if (m_oBehaviorCopy.GetNumberOfCombos() == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "You must specify the trees to which this behavior is applied."));
    }

    //Make sure the path for the harvest executable has been specified
    sData = m_jExecutablePathEdit.getText().trim();
    if (sData.length() == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "You must specify the path to the harvest executable."));
    }

    //Make sure the harvest executable exists
    if (false == new File(sData).exists()) {
      throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "I cannot find the harvest executable."));
    }

    //Make sure the file that SORTIE will write has been specified
    if (m_jSORTIEOutFileEdit.getText().trim().length() == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "You must specify the path to the tree file that SORTIE will write."));
    }

    //Make sure the file that the harvest executable will write has been
    //specified
    if (m_jExecutableHarvestOutFileEdit.getText().trim().length() == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "You must specify the path to the tree harvest file that the executable will write."));
    }

    //Make sure the harvest period has been specified
    if (m_jHarvestPeriod.getText().trim().length() == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "You must specify how often to harvest."));
    }

    //Make sure the harvest period is a positive integer
    try {
      Integer iTest = new Integer(m_jHarvestPeriod.getText().trim());
      if (iTest.intValue() < 0) {
        throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "How often to harvest must be a positive integer value."));
      }
    } catch (java.lang.NumberFormatException e) {
      throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "How often to harvest must be a positive integer value."));
    }

    //Make sure that, if either the batch parameters file or the single-run
    //batch parameters file has been specified, the other has been also
    if ((m_jBatchParamsFileEdit.getText().length() > 0) !=
        (m_jBatchSingleRunParamsFileEdit.getText().length() > 0)) {
      throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "For SORTIE to process batch parameters, both "+
                               "the parameters file for batch run and the " +
                               "single-run parameters file for batch run must "+
                               "be specified."));
    }

    //Make sure that the batch parameters file exists
    if (m_jBatchParamsFileEdit.getText().trim().length() > 0 &&
        new File(m_jBatchParamsFileEdit.getText().trim()).exists() == false) {
      throw(new ModelException(ErrorGUI.BAD_DATA,
                               "JAVA",
                               "The parameters file for batch run does not exist."));
    }

    //If there is an update file specified but no new data members, issue a
    //warning (in the form of a dialog box instead of an error) but let the
    //assignment proceed if the user wishes
    if (m_jExecutableUpdateOutFileEdit.getText().trim().length() > 0 &&
        m_jNewTreeDataMembersListModel.size() == 0)  {
      int iReturn = JOptionPane.showConfirmDialog(this, "Because there are no "+
               "new tree data members being added, the tree update file will "+
               "be ignored.  Continue?", "SORTIE", JOptionPane.YES_NO_OPTION);
      if (iReturn == JOptionPane.NO_OPTION) return false;
    }

    //If there are new tree data members specified but no update file, issue a
    //warning (in the form of a dialog box instead of an error) but let the
    //assignment proceed if the user wishes
    if (m_jExecutableUpdateOutFileEdit.getText().trim().length() == 0 &&
        m_jNewTreeDataMembersListModel.size() > 0)  {
      int iReturn = JOptionPane.showConfirmDialog(this, "Because there is no "+
               "tree update file specified, the new tree data members will not"+
               " be added. Continue?", "SORTIE", JOptionPane.YES_NO_OPTION);
      if (iReturn == JOptionPane.NO_OPTION) return false;
    }

    //*******************
    // Data assignment
    //*******************

    // Species/type combos assigned to this behavior
    Behavior oHarvInt = oDisturbance.GetBehaviorByKey("Harvest Interface");
    oHarvInt.ClearSpeciesTypeCombos();
    for (i = 0; i < m_oBehaviorCopy.GetNumberOfCombos(); i++) {
      oHarvInt.AddSpeciesTypeCombo((SpeciesTypeCombo)m_oBehaviorCopy.GetSpeciesTypeCombo(i).clone());
    }
    oHarvInt.m_bIsEnabled = true;

    // Executable path
    oDisturbance.m_sHarvIntExecutable.SetValue(m_jExecutablePathEdit.getText().trim());

    // Filename that SORTIE is to write
    oDisturbance.m_sHarvIntSORTIEOutFile.SetValue(m_jSORTIEOutFileEdit.getText().trim());

    // Executable's harvest reply file
    oDisturbance.m_sHarvIntExecHarvestOutFile.SetValue(m_jExecutableHarvestOutFileEdit.getText().trim());

    // Harvest period
    oDisturbance.m_iHarvIntHarvestPeriod.SetValue(new Integer(m_jHarvestPeriod.getText().trim()).intValue());

    // Additional file columns
    oDisturbance.mp_sHarvIntFileColumns.GetValue().clear();
    //Start by finding the dashed line
    int iDashedIndex = 0;
    for (i = 0; i < m_jFileColumnsListModel.size(); i++) {
      if (((String)m_jFileColumnsListModel.elementAt(i)).startsWith("--")) {
        iDashedIndex = i+1;
        break;
      }
    }
    for (i = iDashedIndex; i < m_jFileColumnsListModel.size(); i++) {
      oDisturbance.mp_sHarvIntFileColumns.GetValue().add(m_jFileColumnsListModel.elementAt(i));
    }

    // Argument strings to pass to the executable
    oDisturbance.m_sHarvIntExecArgs.SetValue(m_jExecutableArgs.getText().trim());

    // Batch parameters file
    oDisturbance.m_sHarvIntBatchParamsFile.SetValue(m_jBatchParamsFileEdit.getText().trim());

    // File for a single run of batch parameters
    oDisturbance.m_sHarvIntBatchSingleRunParamsFile.SetValue(m_jBatchSingleRunParamsFileEdit.getText().trim());

    // Executable's update reply file - only if there are new tree data members
    if (m_jNewTreeDataMembersListModel.size() > 0) {
      oDisturbance.m_sHarvIntExecUpdateOutFile.SetValue(m_jExecutableUpdateOutFileEdit.getText().trim());
    } else {
      oDisturbance.m_sHarvIntExecUpdateOutFile.SetValue("");
    }

    // New tree data members
    //If there is no reply file indicated, do nothing even if new data members
    //have been added
    if (m_jExecutableUpdateOutFileEdit.getText().trim().length() > 0) {
      oDisturbance.mp_sHarvIntNewTreeDataMembers.clear();
      for (i = 0; i < m_jNewTreeDataMembersListModel.size(); i++) {
        oDisturbance.mp_sHarvIntNewTreeDataMembers.add(m_jNewTreeDataMembersListModel.elementAt(i));
      }
    } else {
      oDisturbance.mp_sHarvIntNewTreeDataMembers.clear();
    }

    return true;
  }


  /**
   * Dialog for choosing new file columns.  This depends on there being some
   * species/type combos for the behavior in order to work correctly.
   * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
   * <p>Company: Institute of Ecosystem Studies</p>
   * @author Lora E. Murphy
   * @version 1.0
   */
  protected class FileColumnsChooser
      extends JDialog
      implements java.awt.event.ActionListener {

    /** List displaying potential file columns */
    protected JList m_jPotentialFileColumnsList;

    /** List model for displaying potential file columns */
    protected DefaultListModel m_jPotentialFileColumnsListModel = new
        DefaultListModel();

    /**
     * Constructor.  Updates the list of potential columns, builds the GUI,
     * and displays.
     * @param jParent Parent window
     * @throws ModelException passed through from called functions
     */
    public FileColumnsChooser(JDialog jParent) throws ModelException {
      super(jParent, "Add File Columns", true);

      //**********************************************************************
       // Get list of data members
       //**********************************************************************

        //First, get a list of all data members to which all species/types apply
        //To make it easier to check, build an array of booleans for species and
        //type
        BehaviorTypeBase[] p_oBehaviorGroups = m_oManager.GetAllBehaviorGroups();
      TreePopulation oPop = m_oManager.GetTreePopulation();
      Vector p_sFoundList = new Vector(0), //data members we've found
          p_bAppliesMatrix = new Vector(0); //boolean matrix for each
      int i, j, k,
          iBehGroup, iBeh, iDM, //loop counters
          iNumSpecies = oPop.GetNumberOfSpecies(),
          iNumTypes = oPop.GetNumberOfTypes();
      boolean[][] p_bBehaviorTrees = new boolean[iNumSpecies][iNumTypes],
          p_bAppliesTo; //what one data member applies to
      boolean bFound;

      for (i = 0; i < iNumSpecies; i++) {
        for (j = 0; j < iNumTypes; j++) {
          p_bBehaviorTrees[i][j] = false;
        }
      }
      for (i = 0; i < m_oBehaviorCopy.GetNumberOfCombos(); i++) {
        SpeciesTypeCombo oCombo = m_oBehaviorCopy.GetSpeciesTypeCombo(i);
        p_bBehaviorTrees[oCombo.GetSpecies()][oCombo.GetType()] = true;
      }

      //Now, go through all data members and find those that apply to all
      //species and types.  To do this, we'll build a boolean array for each
      //one we find, just like the one we did for the behavior.  Data members
      //can be spread out over multiple behaviors.

      //Start with the tree population - the only one we're not already using
      //is "Age" for snags
      p_sFoundList.add("Age");
      p_bAppliesTo = new boolean[iNumSpecies][iNumTypes];
      for (i = 0; i < iNumSpecies; i++) {
        for (j = 0; j < iNumTypes; j++) {
          p_bAppliesTo[i][j] = false;
        }
      }
      for (i = 0; i < iNumSpecies; i++) {
        p_bAppliesTo[i][TreePopulation.SNAG] = true;
      }
      p_bAppliesMatrix.add(p_bAppliesTo);

      for (iBehGroup = 0; iBehGroup < p_oBehaviorGroups.length; iBehGroup++) {
        Behavior[] p_oBehaviors = p_oBehaviorGroups[iBehGroup].GetAllBehaviors();
        if (p_oBehaviors != null) {
          for (iBeh = 0; iBeh < p_oBehaviors.length; iBeh++) {

            //Only collect data member data for enabled behaviors
            if (p_oBehaviors[iBeh].m_bIsEnabled) {
              for (iDM = 0;
                   iDM < p_oBehaviors[iBeh].mp_oNewTreeDataMembers.size(); iDM++) {
                DataMember oDataMember = (DataMember) p_oBehaviors[iBeh].
                    mp_oNewTreeDataMembers.elementAt(iDM);

                //Have we already seen this data member?
                bFound = false;
                for (i = 0; i < p_sFoundList.size(); i++) {
                  if (oDataMember.GetCodeName().equals( (String) p_sFoundList.
                      elementAt(i))) {
                    bFound = true;
                    break;
                  }
                }
                if (bFound) {

                  //Add those tree species/type combos to which it applies
                  p_bAppliesTo = (boolean[][]) p_bAppliesMatrix.elementAt(i);

                  for (j = 0; j < p_oBehaviors[iBeh].GetNumberOfCombos(); j++) {
                    SpeciesTypeCombo oCombo = p_oBehaviors[iBeh].
                        GetSpeciesTypeCombo(j);
                    p_bAppliesTo[oCombo.GetSpecies()][oCombo.GetType()] = true;
                  }

                  p_bAppliesMatrix.remove(i);
                  p_bAppliesMatrix.add(i, p_bAppliesTo);

                }
                else {
                  //This is a newly encountered data member
                  p_sFoundList.add(oDataMember.GetCodeName());

                  //Create a new boolean matrix and add it as well
                  p_bAppliesTo = new boolean[iNumSpecies][iNumTypes];
                  for (j = 0; j < iNumSpecies; j++) {
                    for (k = 0; k < iNumTypes; k++) {
                      p_bAppliesTo[j][k] = false;
                    }
                  }
                  for (j = 0; j < p_oBehaviors[iBeh].GetNumberOfCombos(); j++) {
                    SpeciesTypeCombo oCombo = p_oBehaviors[iBeh].
                        GetSpeciesTypeCombo(j);
                    p_bAppliesTo[oCombo.GetSpecies()][oCombo.GetType()] = true;
                  }

                  p_bAppliesMatrix.add(p_bAppliesTo);
                }

              }
            }
          }
        }
      }

      //Add all user-created data members to the file columns list
      for (i = 0; i < m_jNewTreeDataMembersListModel.size(); i++) {
        m_jPotentialFileColumnsListModel.addElement(
            m_jNewTreeDataMembersListModel.elementAt(i));
      }

      //Now p_sFoundList is a unique list of all the data members for all
      //enabled behaviors in this run; and p_bAppliesMatrix is a list of
      //matrixes for each with those species/types to which they apply.
      //Compare the matrixes of those data members with the matrix for our
      //behavior, and choose the right data members
      boolean bOK;
      for (i = 0; i < p_bAppliesMatrix.size(); i++) {
        bOK = true;
        p_bAppliesTo = (boolean[][]) p_bAppliesMatrix.elementAt(i);
        for (j = 0; j < iNumSpecies; j++) {
          for (k = 0; k < iNumTypes; k++) {
            if (true == p_bBehaviorTrees[j][k] && false == p_bAppliesTo[j][k]) {
              bOK = false;
              break;
            }
          }
        }

        if (bOK) {
          m_jPotentialFileColumnsListModel.addElement( (String) p_sFoundList.
              elementAt(i));
        }
      }

      //**********************************************************************
      // Build GUI
      //**********************************************************************
      if (m_jPotentialFileColumnsListModel.size() == 0) {
        JOptionPane.showMessageDialog(jParent, "No data available to add.");
        return;
      }
      JPanel jContentPane = new JPanel(new BorderLayout());

      //Label
      JLabel jLabel = new JLabel("Select new file columns:");
      jLabel.setFont(new ModelFont());
      jContentPane.add(jLabel, BorderLayout.NORTH);
      m_jPotentialFileColumnsList = new JList(m_jPotentialFileColumnsListModel);
      m_jPotentialFileColumnsList.setFont(new ModelFont());
      JScrollPane jScroller = new JScrollPane(m_jPotentialFileColumnsList);
      jContentPane.add(jScroller, BorderLayout.CENTER);
      jContentPane.add(new OKCancelButtonPanel(this, m_oManager.GetHelpBroker(),
                                               m_sHelpID, true),
                       BorderLayout.SOUTH);
      this.setContentPane(jContentPane);
      this.pack();
      this.setLocationRelativeTo(null);
      this.setVisible(true);
    }

    /**
     * Takes care of button pushes.
     * @param e ActionEvent What event triggered this.
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
      String sCommand = e.getActionCommand();
      if (sCommand.equals("Cancel")) {
        this.setVisible(false);
        this.dispose();
      }
      else if (sCommand.equals("OK")) {

        //Add any selected columns to the list
        Object[] p_sSelected = m_jPotentialFileColumnsList.getSelectedValues();
        String sSelected;
        int i, j;
        boolean bFound;
        for (i = 0; i < p_sSelected.length; i++) {

          sSelected = (String)p_sSelected[i];

          //Make sure it isn't already on the list
          bFound = false;
          for (j = 0; j < m_jFileColumnsListModel.size(); j++) {
            if (sSelected.equals((String)m_jFileColumnsListModel.elementAt(j))) {
              bFound = true;
              break;
            }
          }

          if (false == bFound) {
            m_jFileColumnsListModel.addElement(sSelected);
          }
        }

        this.setVisible(false);
        this.dispose();
      }
    }
  }
}
