package javawrapper;

import junit.framework.*;

/**
 * Tests the WorkerBase class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class TestWorkerBase extends TestCase {

  /** Test object */
  private WorkerTester m_oWorkerBaseTester = null;

  /**
   * Does set up by creating the test object.
   * @throws Exception Won't.
   */
  protected void setUp() throws Exception {
    super.setUp();
    m_oWorkerBaseTester = new WorkerTester();
  }

  /**
   * Does tear down.
   * @throws Exception Won't.
   */
  protected void tearDown() throws Exception {
    m_oWorkerBaseTester = null;
    super.tearDown();
  }

  /**
   * Tests the MakeSureAllAreProportions methods.
   */
  public void testMakeSureAllAreProportions() {

    //Normal processing
    Float fVal;
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector("", "", "", iNumElements,
                                          ModelVector.FLOAT);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {
      //Case - all are proportions and are applied to all
      fVal = new Float(0.11);
      for (i = 0; i < iNumElements; i++) {
        fVal = new Float(fVal.floatValue() + 0.1);
        p_oData.GetValue().add(i, fVal);
        p_bAppliesTo[i] = true;
      }
      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);

      //Case - all are proportions and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);

      //Case - some are applied and some non-proportions exist in unapplied
      //buckets
      p_bAppliesTo[2] = false;
      p_oData.GetValue().remove(2);
      p_oData.GetValue().add(2, new Float(2.3));
      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllAreProportions testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureAllAreProportions normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches non-proportion values that are negative
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData = null;
      p_oData = new ModelVector("", "", "", iNumElements, ModelVector.FLOAT);
      fVal = new Float(0.11);
      for (i = 0; i < iNumElements; i++) {
        fVal = new Float(fVal.floatValue() + 0.1);
        p_oData.GetValue().add(i, fVal);
        p_bAppliesTo[i] = true;
      }

      p_oData.GetValue().remove(2);
      p_oData.GetValue().add(2, new Float( -2.3));

      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);
      fail("MakeSureAllAreProportions failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches non-proportion values that are greater than 1
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData = null;
      p_oData = new ModelVector("", "", "", iNumElements, ModelVector.FLOAT);
      fVal = new Float(0.11);
      for (i = 0; i < iNumElements; i++) {
        fVal = new Float(fVal.floatValue() + 0.1);
        p_oData.GetValue().add(i, fVal);
        p_bAppliesTo[i] = true;
      }

      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Float(2.3));

      m_oWorkerBaseTester.MakeSureAllAreProportions(p_oData, p_bAppliesTo);
      fail("MakeSureAllAreProportions failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllAreProportions exception processing testing succeeded.");
  }

  /**
   * Testing to make sure there are no negative values in a float array.
   */
  public void testMakeSureAllNonNegative() {
    int iNumElements = 4, i;
    Float[] p_fData = new Float[iNumElements];
    String sName = "Float Test";
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-negative and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0);
      p_fData[1] = new Float(2.3);
      p_fData[2] = new Float(0.74);
      p_fData[3] = new Float(5.88);
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_fData, sName, p_bAppliesTo);

      //Case - all are non-negative and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_fData, sName, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_fData[0] = new Float( -0.3);
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_fData, sName, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllNonNegative testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureAllNonNegative normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0);
      p_fData[1] = new Float( -2.3);
      p_fData[2] = new Float(0.74);
      p_fData[3] = new Float(5.88);

      m_oWorkerBaseTester.MakeSureAllNonNegative(p_fData, sName, p_bAppliesTo);
      fail("MakeSureAllNonNegative normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllNonNegative exception processing testing succeeded.");
  }

  /**
   * Testing for MakeSureAllNonZero in ModelVectors
   */
  public void testMakeSureAllNonZero() {
    //Normal processing
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector(
        "ModelVector testing for MakeSureAllNonZero", "", "", iNumElements,
        ModelVector.INTEGER);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-zero and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().add(0, new Integer(5));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(-74));
      p_oData.GetValue().add(3, new Integer(588));
      m_oWorkerBaseTester.MakeSureAllNonZero(p_oData, p_bAppliesTo);

      //Case - all are non-zero and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllNonZero(p_oData, p_bAppliesTo);

      //Case - some are applied and some zeroes exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Integer( 0 ));
      m_oWorkerBaseTester.MakeSureAllNonZero(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllNonZero testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureAllNonZero normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches zero values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(2));
      p_oData.GetValue().add(1, new Integer(0));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer( -588));

      m_oWorkerBaseTester.MakeSureAllNonZero(p_oData, p_bAppliesTo);
      fail("MakeSureAllNonZero normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllNonZero exception processing testing succeeded.");

  }

  /**
   * Testing to make sure there are no negative values in an integer array.
   */
  public void testMakeSureAllNonNegative1() {
    int iNumElements = 4, i;
    Integer[] p_iData = new Integer[iNumElements];
    String sName = "Integer test";
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-negative and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer(0);
      p_iData[1] = new Integer(2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(588);
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_iData, sName, p_bAppliesTo);

      //Case - all are non-negative and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_iData, sName, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_iData[0] = new Integer( -3);
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_iData, sName, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllNonNegative testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureAllNonNegative normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer(0);
      p_iData[1] = new Integer( -2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(588);

      m_oWorkerBaseTester.MakeSureAllNonNegative(p_iData, sName, p_bAppliesTo);
      fail("MakeSureAllNonNegative normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllNonNegative exception processing testing succeeded.");

  }

  /**
   * Testing for non-negatives in ModelVectors
   */
  public void testMakeSureAllNonNegative2() {
    //Normal processing
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector(
        "ModelVector testing for MakeSureAllNonNegative", "", "", iNumElements,
        ModelVector.INTEGER);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-negative and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(588));
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

      //Case - all are non-negative and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Integer( -3));
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllNonNegative testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureAllNonNegative normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer( -588));

      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);
      fail("MakeSureAllNonNegative normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllNonNegative exception processing testing succeeded.");

  }

  /**
   * Testing to make sure all values in a Float array are greater than 0.
   */
  public void testMakeSureAllPositive() {
    int iNumElements = 4, i;
    Float[] p_fData = new Float[iNumElements];
    String sName = "Float Test";
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are positive and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0.66);
      p_fData[1] = new Float(2.3);
      p_fData[2] = new Float(0.74);
      p_fData[3] = new Float(5.88);
      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);

      //Case - all are positive and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_fData[0] = new Float( -0.3);
      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllPositive testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureAllPositive normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0.66);
      p_fData[1] = new Float( -2.3);
      p_fData[2] = new Float(0.74);
      p_fData[3] = new Float(5.88);

      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches 0 values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_fData[0] = new Float(0.66);
      p_fData[1] = new Float(2.3);
      p_fData[2] = new Float(0);
      p_fData[3] = new Float(5.88);

      m_oWorkerBaseTester.MakeSureAllPositive(p_fData, sName, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }
    System.out.println(
        "MakeSureAllPositive exception processing testing succeeded.");
  }

  /**
   * Testing to make sure all values in a vector are between two bounds.
   */
  public void testMakeSureAllAreBounded() {
    //Normal processing
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector(
        "ModelVector testing for MakeSureAllAreBounded", "", "", iNumElements,
        ModelVector.INTEGER);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are between the bounds and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(100));
      m_oWorkerBaseTester.MakeSureAllAreBounded(p_oData, p_bAppliesTo, 0, 100);

      //Case - all are between the bounds and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);

      //Case - some are applied and some out-of-bounds exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Integer( -3));
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllAreBounded testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureAllAreBounded normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches below-lower-bound values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(1));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer( -588));

      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);
      fail("MakeSureAllAreBounded failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches above-upper-bound values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(588));

      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);
      fail("MakeSureAllAreBounded normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllAreBounded exception processing testing succeeded.");
  }

  /**
   * Tests to make sure all values in an Integer array are greater than 0.
   */
  public void testMakeSureAllPositive1() {
    int iNumElements = 4, i;
    Integer[] p_iData = new Integer[iNumElements];
    String sName = "Integer Test";
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are positive and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer(66);
      p_iData[1] = new Integer(2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(588);
      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);

      //Case - all are positive and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_iData[0] = new Integer( -3);
      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllPositive testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureAllPositive normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer( -66);
      p_iData[1] = new Integer(2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(588);

      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches 0 values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_iData[0] = new Integer(66);
      p_iData[1] = new Integer(2);
      p_iData[2] = new Integer(74);
      p_iData[3] = new Integer(0);

      m_oWorkerBaseTester.MakeSureAllPositive(p_iData, sName, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }
    System.out.println(
        "MakeSureAllPositive exception processing testing succeeded.");

  }

  /**
   * Tests MakeSureAllPositive methods.
   */
  public void testMakeSureAllPositive2() {
    //Normal processing
    int iNumElements = 4, i;
    ModelVector p_oData = new ModelVector(
        "ModelVector testing for MakeSureAllPositive", "", "", iNumElements,
        ModelVector.INTEGER);
    boolean[] p_bAppliesTo = new boolean[iNumElements];

    try {

      //Case - all are non-negative and are applied to all
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().add(0, new Integer(1));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(588));
      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);

      //Case - all are non-negative and none are applied
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = false;
      }
      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);

      //Case - some are applied and some negatives exist in unapplied
      //buckets
      p_bAppliesTo[0] = false;
      p_oData.GetValue().remove(0);
      p_oData.GetValue().add(0, new Integer( -3));
      m_oWorkerBaseTester.MakeSureAllNonNegative(p_oData, p_bAppliesTo);

    }
    catch (ModelException oErr) {
      fail("MakeSureAllPositive testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureAllPositive normal processing testing succeeded.");

    //Exception processing
    try {

      //Catches negative values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(1));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer( -588));

      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {

      //Catches 0 values
      for (i = 0; i < iNumElements; i++) {
        p_bAppliesTo[i] = true;
      }
      p_oData.GetValue().clear();
      p_oData.GetValue().add(0, new Integer(0));
      p_oData.GetValue().add(1, new Integer(2));
      p_oData.GetValue().add(2, new Integer(74));
      p_oData.GetValue().add(3, new Integer(588));

      m_oWorkerBaseTester.MakeSureAllPositive(p_oData, p_bAppliesTo);
      fail("MakeSureAllPositive normal failed to catch invalid values.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureAllPositive exception processing testing succeeded.");

  }

  /**
   * Tests to make sure a ModelFloat value is a proportion.
   */
  public void testMakeSureIsProportion() {
    ModelFloat fValue = new ModelFloat( (float) 0.45,
                                       "ModelFloat proportion test", "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureIsProportion(fValue);
    }
    catch (ModelException oErr) {
      fail("MakeSureIsProportion testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureIsProportion normal processing testing succeeded.");

    //Exception processing
    try {
      fValue.SetValue( (float) - 0.23);
      m_oWorkerBaseTester.MakeSureIsProportion(fValue);
      fail("MakeSureIsProportion failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      fValue.SetValue( (float) 23);
      m_oWorkerBaseTester.MakeSureIsProportion(fValue);
      fail("MakeSureIsProportion failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureIsProportion exception processing testing succeeded.");
  }

  /**
   * Tests to make sure a ModelFloat value is not negative
   */
  public void testMakeSureNonNegative() {
    ModelFloat fValue = new ModelFloat( (float) 0.45,
                                       "ModelFloat MakeSureNonNegative test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureNonNegative(fValue);
    }
    catch (ModelException oErr) {
      fail("MakeSureNonNegative testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureNonNegative normal processing testing succeeded.");

    //Exception processing
    try {
      fValue.SetValue( (float) - 0.23);
      m_oWorkerBaseTester.MakeSureNonNegative(fValue);
      fail("MakeSureNonNegative failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureNonNegative exception processing testing succeeded.");
  }


  /**
   * Tests to make sure a ModelInt value is not negative
   */
  public void testMakeSureNonNegative1() {
    ModelInt iValue = new ModelInt(2, "ModelInt MakeSureNonNegative test", "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureNonNegative(iValue);
    }
    catch (ModelException oErr) {
      fail("MakeSureNonNegative testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSureNonNegative normal processing testing succeeded.");

    //Exception processing
    try {
      iValue.SetValue( -23);
      m_oWorkerBaseTester.MakeSureNonNegative(iValue);
      fail("MakeSureNonNegative failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureNonNegative exception processing testing succeeded.");

  }

  /**
   * Tests to make sure a ModelFloat value is not equal to zero
   */
  public void testMakeSureNonZero1() {
    ModelFloat fValue = new ModelFloat( (float) 0.45,
                                       "ModelFloat MakeSureNonZero test",
                                       "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureNonZero(fValue);
    }
    catch (ModelException oErr) {
      fail("MakeSureNonZero testing failed with message " +
           oErr.sMessage);
    }

    fValue.SetValue((float)-0.2);
    try {
      m_oWorkerBaseTester.MakeSureNonZero(fValue);
    }
    catch (ModelException oErr) {
      fail("MakeSureNonZero testing failed with message " +
           oErr.sMessage);
    }

    System.out.println(
        "MakeSureNonZero normal processing testing succeeded.");

    //Exception processing
    try {
      fValue.SetValue( (float) 0);
      m_oWorkerBaseTester.MakeSureNonZero(fValue);
      fail("MakeSureNonZero failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureNonZero exception processing testing succeeded.");
  }

  /**
   * Tests to make sure a ModelInt value is not equal to zero
   */
  public void testMakeSureNonZero2() {
    ModelInt iValue = new ModelInt(5, "ModelInt MakeSureNonZero test", "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSureNonZero(iValue);
    }
    catch (ModelException oErr) {
      fail("MakeSureNonZero testing failed with message " +
           oErr.sMessage);
    }

    iValue.SetValue(-2);
    try {
      m_oWorkerBaseTester.MakeSureNonZero(iValue);
    }
    catch (ModelException oErr) {
      fail("MakeSureNonZero testing failed with message " +
           oErr.sMessage);
    }

    System.out.println(
        "MakeSureNonZero normal processing testing succeeded.");

    //Exception processing
    try {
      iValue.SetValue(0);
      m_oWorkerBaseTester.MakeSureNonZero(iValue);
      fail("MakeSureNonZero failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSureNonZero exception processing testing succeeded.");
  }

  /**
   * Tests to make sure a ModelFloat value is greater than zero.
   */
  public void testMakeSurePositive() {
    ModelFloat fValue = new ModelFloat( (float) 45,
                                       "ModelFloat MakeSurePositive test", "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSurePositive(fValue);
    }
    catch (ModelException oErr) {
      fail("MakeSurePositive testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSurePositive normal processing testing succeeded.");

    //Exception processing
    try {
      fValue.SetValue( (float) - 0.23);
      m_oWorkerBaseTester.MakeSurePositive(fValue);
      fail("MakeSurePositive failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }
    try {
      fValue.SetValue( (float) 0);
      m_oWorkerBaseTester.MakeSurePositive(fValue);
      fail("MakeSurePositive failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSurePositive exception processing testing succeeded.");
  }

  /**
   * Tests to make sure a ModelInt value is greater than zero.
   */
  public void testMakeSurePositive1() {
    ModelInt iValue = new ModelInt(545, "ModelInt MakeSurePositive test", "");
    //Normal processing
    try {
      m_oWorkerBaseTester.MakeSurePositive(iValue);
    }
    catch (ModelException oErr) {
      fail("MakeSurePositive testing failed with message " +
           oErr.sMessage);
    }
    System.out.println(
        "MakeSurePositive normal processing testing succeeded.");

    //Exception processing
    try {
      iValue.SetValue( -23);
      m_oWorkerBaseTester.MakeSurePositive(iValue);
      fail("MakeSurePositive failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }
    try {
      iValue.SetValue(0);
      m_oWorkerBaseTester.MakeSurePositive(iValue);
      fail("MakeSurePositive failed to catch invalid value.");
    }
    catch (ModelException oErr) {
      ;
    }

    System.out.println(
        "MakeSurePositive exception processing testing succeeded.");

  }

  /**
   * Tests change of species
   */
  public void testChangeOfSpecies() {
    try {

      //Old number of species
      int iOldNumberOfSpecies = 4, i;

      //New species indexer
      int[] p_oIndexer = new int[5];
      p_oIndexer[0] = -1; //First species in new list is new
      p_oIndexer[1] = 1; //Second species is the same from old to new
      p_oIndexer[2] = 3; //Third species in new list is same as fourth in old
      p_oIndexer[3] = 0; //Fourth species in new list is same as first in old
      p_oIndexer[4] = -1; //Fifth species in new list is old

      ModelVector
          //Should not get touched - too short
          p_oNonSpeciesSpecificTooShort = new ModelVector("Too short",
          "", "", 0, ModelVector.FLOAT),
          //Should not get touched - too long
          p_oNonSpeciesSpecificTooLong = new ModelVector("Too long", "", "", 0,
          ModelVector.STRING),
          //Species-specific set of strings
          p_oStringSpeciesSpecific = new ModelVector("String", "", "", 0,
          ModelVector.STRING),
          //Species-specific set of floats
          p_oFloatSpeciesSpecific = new ModelVector("Float", "", "", 0,
          ModelVector.FLOAT),
          //Species-specific set of ModelEnums
          p_oModelEnumSpeciesSpecific = new ModelVector("ModelEnum", "", "", 0,
          ModelVector.MODEL_ENUM),
          //Species-specific set of integers
          p_oIntegerSpeciesSpecific = new ModelVector("ModelInteger", "", "", 0,
          ModelVector.INTEGER);

      //Give all the vectors data
      p_oNonSpeciesSpecificTooShort.GetValue().add(new Float(1));
      p_oNonSpeciesSpecificTooShort.GetValue().add(new Float(2));

      p_oNonSpeciesSpecificTooLong.GetValue().add("test 1");
      p_oNonSpeciesSpecificTooLong.GetValue().add("test 2");
      p_oNonSpeciesSpecificTooLong.GetValue().add("test 3");
      p_oNonSpeciesSpecificTooLong.GetValue().add("test 4");
      p_oNonSpeciesSpecificTooLong.GetValue().add("test 5");

      p_oStringSpeciesSpecific.GetValue().add("species 0");
      p_oStringSpeciesSpecific.GetValue().add("species 1");
      p_oStringSpeciesSpecific.GetValue().add("species 2");
      p_oStringSpeciesSpecific.GetValue().add("species 3");

      p_oFloatSpeciesSpecific.GetValue().add(new Float(0));
      p_oFloatSpeciesSpecific.GetValue().add(new Float(1));
      p_oFloatSpeciesSpecific.GetValue().add(new Float(2));
      p_oFloatSpeciesSpecific.GetValue().add(new Float(3));

      p_oModelEnumSpeciesSpecific.GetValue().add(new ModelEnum(null, null,
          "enum 0", ""));
      p_oModelEnumSpeciesSpecific.GetValue().add(new ModelEnum(null, null,
          "enum 1", ""));
      p_oModelEnumSpeciesSpecific.GetValue().add(new ModelEnum(null, null,
          "enum 2", ""));
      p_oModelEnumSpeciesSpecific.GetValue().add(new ModelEnum(null, null,
          "enum 3", ""));

      p_oIntegerSpeciesSpecific.GetValue().add(new Integer(0));
      p_oIntegerSpeciesSpecific.GetValue().add(new Integer(1));
      p_oIntegerSpeciesSpecific.GetValue().add(new Integer(2));
      p_oIntegerSpeciesSpecific.GetValue().add(new Integer(3));

      //Put the data in the test object
      WorkerBase oTest = new WorkerTester();
      //ModelInt - should be ignored
      oTest.mp_oAllData.add(new ModelInt(1, "test", "test"));
      oTest.mp_oAllData.add(p_oIntegerSpeciesSpecific);
      oTest.mp_oAllData.add(p_oNonSpeciesSpecificTooShort);
      oTest.mp_oAllData.add(p_oStringSpeciesSpecific);
      oTest.mp_oAllData.add(p_oNonSpeciesSpecificTooLong);
      oTest.mp_oAllData.add(p_oModelEnumSpeciesSpecific);
      oTest.mp_oAllData.add(p_oFloatSpeciesSpecific);

      oTest.ChangeOfSpecies(iOldNumberOfSpecies, p_oIndexer);

      //Validate each one
      assertEquals(2, p_oNonSpeciesSpecificTooShort.GetValue().size());
      Float fTest = (Float) p_oNonSpeciesSpecificTooShort.GetValue().elementAt(
          0);
      assertEquals(new Float(1), fTest);
      fTest = (Float) p_oNonSpeciesSpecificTooShort.GetValue().elementAt(1);
      assertEquals(new Float(2), fTest);

      assertEquals(5, p_oNonSpeciesSpecificTooLong.GetValue().size());
      String sTest = (String) p_oNonSpeciesSpecificTooLong.GetValue().elementAt(
          0);
      assertEquals("test 1", sTest);
      sTest = (String) p_oNonSpeciesSpecificTooLong.GetValue().elementAt(1);
      assertEquals("test 2", sTest);
      sTest = (String) p_oNonSpeciesSpecificTooLong.GetValue().elementAt(2);
      assertEquals("test 3", sTest);
      sTest = (String) p_oNonSpeciesSpecificTooLong.GetValue().elementAt(3);
      assertEquals("test 4", sTest);
      sTest = (String) p_oNonSpeciesSpecificTooLong.GetValue().elementAt(4);
      assertEquals("test 5", sTest);

      assertEquals(5, p_oStringSpeciesSpecific.GetValue().size());
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(0);
      assertEquals(null, sTest);
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(1);
      assertEquals("species 1", sTest);
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(2);
      assertEquals("species 3", sTest);
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(3);
      assertEquals("species 0", sTest);
      sTest = (String) p_oStringSpeciesSpecific.GetValue().elementAt(4);
      assertEquals(null, sTest);

      assertEquals(5, p_oFloatSpeciesSpecific.GetValue().size());
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(0);
      assertEquals(null, fTest);
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(1);
      assertEquals(new Float(1), fTest);
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(2);
      assertEquals(new Float(3), fTest);
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(3);
      assertEquals(new Float(0), fTest);
      fTest = (Float) p_oFloatSpeciesSpecific.GetValue().elementAt(4);
      assertEquals(null, fTest);

      assertEquals(5, p_oModelEnumSpeciesSpecific.GetValue().size());
      ModelEnum oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().
          elementAt(0);
      assertEquals(null, fTest);
      oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().elementAt(
          1);
      assertTrue(oEnumTest.equals(new ModelEnum(null, null, "enum 1", "")));
      oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().elementAt(
          2);
      assertTrue(oEnumTest.equals(new ModelEnum(null, null, "enum 3", "")));
      oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().elementAt(
          3);
      assertTrue(oEnumTest.equals(new ModelEnum(null, null, "enum 0", "")));
      oEnumTest = (ModelEnum) p_oModelEnumSpeciesSpecific.GetValue().elementAt(
          4);
      assertEquals(null, fTest);

      assertEquals(5, p_oIntegerSpeciesSpecific.GetValue().size());
      Integer iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(
          0);
      assertEquals(null, iTest);
      iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(1);
      assertEquals(new Integer(1), iTest);
      iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(2);
      assertEquals(new Integer(3), iTest);
      iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(3);
      assertEquals(new Integer(0), iTest);
      iTest = (Integer) p_oIntegerSpeciesSpecific.GetValue().elementAt(4);
      assertEquals(null, iTest);

      System.out.println("ChangeOfSpecies testing succeeded.");
    } catch (ModelException oErr) {
      fail("ChangeOfSpecies testing failed with message " + oErr.getMessage());
    }
  }
}

/**
 * Test class for WorkerBase.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
class WorkerTester
    extends WorkerBase {

  /**
   * Constructor.
   */
  WorkerTester() {
    super(null, "Test", null);
  }

  /**
   * Does nothing
   * @param oPop TreePopulation Ignored
   */
  public void DoSetup(TreePopulation oPop) {;}

  /**
   * Overridden abstract function
   * @param oPop Not used
   * @throws ModelException never
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {};

  /**
   * Overridden abstract function
   * @param oPop Not used
   * @param out Not used
   * @throws ModelException never
   */
  public void WriteXML(java.io.FileWriter out, TreePopulation oPop) throws
      ModelException {};
}
